import os
import jinja2
import glob
import shutil
import logging
import traceback
import sys
import re

from pentagon.helpers import render_template
from pentagon.defaults import AWSPentagonDefaults as PentagonDefaults


class ComponentBase(object):
    """ Base class for Pentagon Components. """
    _required_parameters = []

# List of environment variables to use.
# If set, they should override other data sources.
# Lower Case here will find upper case environment variables.
# If a dictionary is passed, the key is the variable name used in context,
# and the value is the environment variable name.
    _environment = []
    _defaults = {}

    def __init__(self, data, additional_args=None, **kwargs):

        self._data = data
        self._additional_args = additional_args
        self._process_env_vars()
        self._process_defaults()

        missing_parameters = []
        for item in self._required_parameters:
            if item not in self._data.keys():
                missing_parameters.append(item)

        if missing_parameters:
            logging.error("Missing required data parameters: {}".format(",".join(missing_parameters)))
            sys.exit(1)

    @property
    def _destination_directory_name(self):
        if self._destination != './':
            return self._destination
        return self._data.get('name', self.__class__.__name__.lower())

    @property
    def _files_directory(self):
        return sys.modules[self.__module__].__path__[0] + "/files"

    def _process_env_vars(self):
        logging.debug('Fetching environment variables')
        environ_data = {}
        for item in self._environment:
            if type(item) is dict:
                context_var = item.keys()[0]
                env_var = os.environ.get(item.values()[0])
            else:
                context_var = item.lower()
                env_var = os.environ.get(item.upper())

            environ_data[context_var] = env_var

        self._merge_data(environ_data)

    def _process_defaults(self):
        """ Use _defaults from global pentagon defaults, then class and add them to missing values on the _data dict """

        logging.debug('Processing Defaults')
        self._merge_data(self._defaults)

        try:
            class_name = self.__class__.__name__.lower()
            pentagon_defaults = getattr(PentagonDefaults, class_name)
            logging.debug("Adding Pentagon Defaults Last {}".format(pentagon_defaults))
            self._merge_data(pentagon_defaults)
        except AttributeError, e:
            logging.info("No top level defaults for Pentagon component {} ".format(class_name.lower()))

    def _render_directory_templates(self):
        """ Loop and use render_template helper method on all templates in destination directory  """
        template_location = self._destination_directory_name
        if os.path.isfile(self._destination_directory_name):
            template_location = os.path.dirname(self._destination_directory_name)
            logging.debug("{} is a file. Using the directory {} instead.".format(self._destination_directory_name, template_location))
        logging.debug("Rendering Templates in {}".format(template_location))
        for folder, dirnames, files in os.walk(template_location):
            for template in glob.glob(folder + "/*.jinja"):
                logging.debug("Rendering {}".format(template))
                template_file_name = template.split('/')[-1]
                path = '/'.join(template.split('/')[0:-1])
                target_file_name = re.sub(r'\.jinja$', '', template_file_name)
                target = folder + "/" + target_file_name
                render_template(template_file_name, path, target, self._data, overwrite=self._overwrite)

    def _remove_init_file(self):
        """ delete init file, if it exists from template target directory """

        for root, dirs, files in os.walk(self._destination_directory_name):
            for name in files:
                if "__init__.py" == name:
                    logging.debug('Removing: {}'.format(os.path.join(root, name)))
                    os.remove(os.path.join(root, name))

    def _merge_data(self, new_data, clobber=False):
        """ accepts new_data (dict) and clobbber (boolean). Merges dictionary with existing instance dictionary _data. If clobber is True, overwrites value. Defaults to false """
        for key, value in new_data.items():
            if self._data.get(key) is None or clobber:
                logging.debug("Setting component data {}: {}".format(key, value))
                self._data[key] = value

    def add(self, destination, overwrite=False):
        self._destination = destination
        self._overwrite = overwrite
        try:
            self._add_files()
            self._remove_init_file()
            self._render_directory_templates()
            logging.info("New component added. Source your environment before proceeding or unexpected behavior may result.")
        except Exception as e:
            logging.error("Error occured configuring component")
            logging.error(e)
            logging.debug(traceback.format_exc(e))
            sys.exit(1)

    def _add_files(self, sub_path=None):
        """ Copies files and templates from <component>/files """
        if self._overwrite:
            from distutils.dir_util import copy_tree
        else:
            from shutil import copytree as copy_tree
        if sub_path is not None:
            source = ('{}/{}').format(self._files_directory, sub_path)
        else:
            source = self._files_directory

        logging.debug("Adding file: {} -> {}".format(source, self._destination_directory_name))
        if os.path.isfile(source):
            shutil.copy(source, self._destination_directory_name)
        elif os.path.isdir(source):
            copy_tree(source, self._destination_directory_name)



import os
import json
import sys
import logging
import traceback

from pentagon.component import ComponentBase
from pentagon.component.aws_vpc import AWSVpc as Vpc
from pentagon.component.vpn import Vpn
from pentagon.helpers import create_rsa_key
from pentagon.defaults import AWSPentagonDefaults as PentagonDefaults


class Inventory(ComponentBase):

    _defaults = {'cloud': 'aws'}
    _required_parameters = [
        'name',
        'infrastructure_bucket',
        'aws_access_key',
        'aws_secret_key',
        'project_name'
    ]

    def __init__(self, data, additional_args=None, **kwargs):
        super(Inventory, self).__init__(data, additional_args, **kwargs)
        self._ssh_keys = {
            'admin_vpn_key': self._data.get('admin_vpn_key', PentagonDefaults.ssh['admin_vpn_key']),
            'working_kube_key': self._data.get('working_kube_key', PentagonDefaults.ssh['working_kube_key']),
            'production_kube_key': self._data.get('production_kube_key', PentagonDefaults.ssh['production_kube_key']),
            'working_private_key': self._data.get('working_private_key', PentagonDefaults.ssh['working_private_key']),
            'production_private_key': self._data.get('production_private_key', PentagonDefaults.ssh['production_private_key']),
        }

    @property
    def _files_directory(self):
        return sys.modules[self.__module__].__path__[0] + "/files/common"

    def add(self, destination, overwrite=False):
        """Inventory version of Component.add Copies files and templates from <component>/files and templates the *.jinja files """
        if destination == './':
            self._destination = self._data.get('name', './default')
        else:
            self._destination = destination

        self._overwrite = overwrite

        try:
            self._add_files()

            if self._data['cloud'].lower() == 'aws':
                self._data['aws_region'] = self._data.get('aws_default_region')
                self._data['account'] = os.path.basename(self._destination)
                self._merge_data(self._ssh_keys)
                self.__create_keys()

                Aws(self._data).add("{}/terraform".format(self._destination))
                if self._data.get('configure_vpn', True):
                    Vpn(self._data).add("{}/resources".format(self._destination), overwrite=True)

            if self._data['cloud'].lower() == 'gcp':
                Gcp(self._data).add(self._destination)

            self._remove_init_file()
            self._render_directory_templates()
        except Exception as e:
            logging.error("Error occured configuring component")
            logging.error(e)
            logging.debug(traceback.format_exc(e))
            sys.exit(1)

    def __create_keys(self):
        key_path = "{}/{}/".format(self._destination, "config/private")

        for key in self._ssh_keys:
            logging.debug("Creating ssh key {}".format(key))
            key_name = "{}".format(self._ssh_keys[key])
            if not os.path.isfile("{}{}".format(key_path, key_name)):
                create_rsa_key(key_name, key_path)
            else:
                logging.warn("Key {}{} exist!".format(key_path, key_name))


class Aws(ComponentBase):

    def add(self, destination):
        Vpc(self._data).add("./{}".format(destination), overwrite=True)


class Gcp(ComponentBase):

    def add(self, destination):
        pass



import os
import json
import sys

from pentagon.component import ComponentBase
from googleapiclient.discovery import build


class Cluster(ComponentBase):

    @property
    def _files_directory(self):
        return sys.modules[self.__module__].__path__[0] + "/files/cluster"

    def get(self, destination):
        service = build('container', 'v1')
        cl = service.projects().zones().clusters()
        self._zone = self._data.get('zone')
        self._project = self._data.get('project')
        self._destination = destination
        cluster_name = self._data.get('name')
        if cluster_name:
            cluster = cl.get(projectId=self._project, zone=self._zone, clusterId=cluster_name).execute()
            Cluster(self.parse_cluster(cluster)).add(cluster_name)
            self.add_cluster_nodepools(cluster_name, cluster.get('nodePools'))
        else:
            clusters = cl.list(projectId=self._project, zone=self._zone).execute()

            for cluster in clusters['clusters']:
                Cluster(self.parse_cluster(cluster)).add(cluster['name'])
                self.add_cluster_nodepools(cluster.get('name'), cluster.get('nodePools'), self._data.get('get_default_nodepools'))

    def parse_cluster(self, cluster):
        """ parses cluster object returned from api call and converts it to a data structure for the .add() method """

        # remove master zone from additional-zones
        locations = cluster.get('locations')
        locations.remove(self._zone)

        for np in cluster.get('nodePools'):
            if np.get('name') == 'default-pool':
                default_nodepool = np

        return {
                    'project': self._project,
                    'zone': self._zone,
                    'cluster_name': cluster['name'],
                    'cluster_cidr': cluster['clusterIpv4Cidr'],
                    'cluster_version': cluster['currentMasterVersion'],
                    'additional_zones': locations,
                    'disk_size': cluster['nodeConfig']['diskSizeGb'],
                    'max_nodes_per_pool': cluster.get('maxNodesPerPool', None),
                    'image_type': cluster['nodeConfig']['imageType'],
                    'node_labels': cluster.get('nodeLabels', []),
                    'num_nodes': cluster['currentNodeCount'],
                    'image_type': cluster['nodeConfig']['imageType'],
                    'username': cluster['masterAuth']['username'],
                    'network': cluster['network'],
                    'subnetwork': cluster.get('subnetwork', None),
                    'scopes': cluster['nodeConfig']['oauthScopes'],
                    'enable_autoscaling': default_nodepool.get('autoscaling'),
                    'min_nodes': default_nodepool.get('autoscaling', {}).get('minNodeCount'),
                    'max_nodes': default_nodepool.get('autoscaling', {}).get('maxNodeCount'),
                }

    def add_cluster_nodepools(self, cluster_name, nodepools, get_default_nodepools=False):
        """ parses cluster node pools and adds nodepool objects """
        for nodepool in nodepools:
            if nodepool['name'] != 'default-pool' or get_default_nodepools:
                node_data = {'cluster': cluster_name, 'zone': self._zone, 'name': nodepool['name'], 'project': self._project}
                Nodepool(node_data).get(self._destination + '/' + cluster_name + '/nodepools')


class Nodepool(ComponentBase):

    _required_parameters = ['cluster']

    @property
    def _files_directory(self):
        return sys.modules[self.__module__].__path__[0] + "/files/nodepool"

    def get(self, destination):
        service = build('container', 'v1')
        np = service.projects().zones().clusters().nodePools()

        self._name = self._data.get('name')
        self._zone = self._data.get('zone')
        self._project = self._data.get('project')
        self._cluster = self._data.get('cluster')

        if self._data.get('name'):
            pool = np.get(
                            projectId=self._project,
                            zone=self._zone,
                            clusterId=self._cluster,
                            nodePoolId=self._name
                            ).execute()
            Nodepool(self.parse_nodepool(pool)).add(destination + '/' + self._name)

        else:
            nodepools = np.list(
                            projectId=self._project,
                            zone=self._zone,
                            clusterId=self._cluster
                            ).execute()

            for pool in nodepools['nodePools']:
                    Nodepool(self.parse_nodepool(pool)).add(destination + '/' + self._name)

    def parse_nodepool(self, pool):
        """ parses nodepool object returned from api call and converts it to a data structure for the .add() method """        
        return {
                    'nodepool_name': self._name,
                    'project': self._project,
                    'zone': self._zone,
                    'cluster': self._cluster,
                    'disk_size': pool.get('config', {}).get('diskSizeGb'),
                    'image_type': pool.get('config', {}).get('imageType'),
                    'machine_type': pool.get('config', {}).get('machineType'),
                    'node_labels': pool.get('nodeLabels'),
                    'num_nodes': pool.get('numNodes'),
                    'enable_autoscaling': pool.get('autoscaling', {}).get('enabled'),
                    'min_nodes': pool.get('autoscaling', {}).get('minNodeCount'),
                    'max_nodes': pool.get('autoscaling', {}).get('maxNodeCount'),
                    'scopes': pool.get('autoscaling', {}).get('oauthScopes')
                }


class Project(ComponentBase):
    def add(self):
        logging.error("Project component is not yet implemented")
        sys.exit(1)



from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict


class Migration(migration.Migration):
    _starting_version = '2.3.1'
    _ending_version = '2.4.0'

    def run(self):
        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            logging.debug(
                'Processing Inventory Item: {}'
                .format(inventory_path)
            )
            with self.YamlEditor('{}/config/local/vars.yml'.format(inventory_path)) as vars_yml:
                vars_yml_dict = vars_yml.get_data()

            logging.info('Found KUBECONFIG in vars.yml = {}'
                         .format(vars_yml_dict.get('KUBECONFIG')))
            logging.info('Found ANSIBLE_CONFIGin vars.yml = {}'
                         .format(vars_yml_dict.get('ANSIBLE_CONFIG')))
            vars_yml_dict['KUBECONFIG'] = '${INFRASTRUCTURE_REPO}/inventory/${INVENTORY}/config/private/kube_config'
            vars_yml_dict['ANSIBLE_CONFIG'] = '${INFRASTRUCTURE_REPO}/inventory/${INVENTORY}/config/private/ansible.cfg'
            logging.info('Changed KUBECONFIG to be {}'
                         .format(vars_yml_dict.get('KUBECONFIG')))
            logging.info('Changed ANSIBLE_CONFIG to be {}'
                         .format(vars_yml_dict.get('ANSIBLE_CONFIG')))

            logging.warn(
                '####### IMPORTANT: Your kube and ansible config paths have changed.')

            vars_yml.write()




import os
import logging
import boto3

from pentagon.component import ComponentBase


class Vpn(ComponentBase):

    _required_parameters = [
        'aws_access_key',
        'aws_secret_key',
        'project_name'
    ]

    _ami_owners = ['099720109477']  # Amazon AMI owner
    _vpn_ami_id_placeholder = "<ami_id>"
    _vpn_ami_filters = [{'Name': 'virtualization-type', 'Values': ['hvm']},
                        {'Name': 'architecture', 'Values': ['x86_64']},
                        {'Name': 'name', 'Values': ['ubuntu/images/hvm-ssd/ubuntu-trusty*']}]

    def add(self, destination, overwrite=False):
        self._get_vpn_ami_id()
        return super(Vpn, self).add(destination, overwrite=overwrite)

    def _get_vpn_ami_id(self):

        if self._data.get('vpn_ami_id'):
            self._data['vpn_ami_id'] = self._data.get('vpn_ami_id')
        else:
            logging.info("Getting VPN ami-id from AWS")

            try:
                client = boto3.client('ec2',
                                      aws_access_key_id=self._data['aws_access_key'],
                                      aws_secret_access_key=self._data['aws_secret_key'],
                                      region_name=self._data['aws_default_region']
                                      )
                images = client.describe_images(Owners=self._ami_owners, Filters=self._vpn_ami_filters)
                self._data['vpn_ami_id'] = images['Images'][-1]['ImageId']
            except Exception, e:
                logging.error("Encountered \" {} \" getting ami-id. VPN not configured fully. See docs/vpn.md for more information".format(e))



from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict

import re


class Migration(migration.Migration):
    _starting_version = '2.5.0'
    _ending_version = '2.6.0'

    def run(self):
        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            logging.debug(
                'Processing Inventory Item: {}'
                .format(inventory_path)
            )

            # Update version of VPC TF module
            aws_vpc_file = "{}/terraform/aws_vpc.tf".format(inventory_path)
            if os.path.exists(aws_vpc_file):
                aws_vpc_file_content = self.get_file_content(aws_vpc_file)
                aws_vpc_file_content = re.sub(r'terraform-vpc.git\?ref=v\d+\.\d+.\d+', 'terraform-vpc.git?ref=v3.0.0', aws_vpc_file_content)
                aws_vpc_file_content = re.sub(r'\n\s*aws_secret_key\s+=.+', '', aws_vpc_file_content)
                aws_vpc_file_content = re.sub(r'\n\s*aws_access_key\s+=.+', '', aws_vpc_file_content)
                self.overwrite_file(aws_vpc_file, aws_vpc_file_content)
                logging.info('Terraform VPC module updated to 3.0.0 in {}'.format(item))

            # Remove TF AWS provider variables from secrets. No longer referenced directly in VPC module.
            secret_file = "{}/config/private/secrets.yml".format(inventory_path)
            if os.path.exists(secret_file):
                secrets_file_content = self.get_file_content(secret_file)
                original_secrets_content = secrets_file_content
                secrets_file_content = re.sub(r'# Terraform.*\n', '', secrets_file_content)
                secrets_file_content = re.sub(r'TF_VAR_aws_secret_key:.*\n', '', secrets_file_content)
                secrets_file_content = re.sub(r'TF_VAR_aws_access_key:.*\n\n?', '', secrets_file_content)
                self.overwrite_file(secret_file, secrets_file_content)
                
                if original_secrets_content != secrets_file_content:
                    logging.warn("####### IMPORTANT: Secrets file has been updated #######")
                    logging.warn("  Update changed secrets file in 1Password: {}".format(secret_file))
                    logging.warn("  Terraform AWS provider variables removed in VPC module update and no longer needed in secrets.")



import os

from pentagon.component import ComponentBase
from pentagon.defaults import AWSPentagonDefaults as PentagonDefaults
from pentagon.helpers import allege_aws_availability_zones


class AWSVpc(ComponentBase):

    _required_parameters = ['aws_region']

    def add(self, destination, overwrite):
        for key, value in PentagonDefaults.vpc.iteritems():
            if not self._data.get(key):
                self._data[key] = value

        if self._data.get('aws_availability_zones') is None:
            self._data['aws_availability_zones'] = allege_aws_availability_zones(self._data['aws_region'], self._data['aws_availability_zone_count'])

        return super(AWSVpc, self).add(destination, overwrite=overwrite)



#!/usr/bin/env python

'''
EC2 external inventory script
=================================

Generates inventory that Ansible can understand by making API request to
AWS EC2 using the Boto library.

NOTE: This script assumes Ansible is being executed where the environment
variables needed for Boto have already been set:
    export AWS_ACCESS_KEY_ID='AK123'
    export AWS_SECRET_ACCESS_KEY='abc123'

This script also assumes there is an ec2.ini file alongside it.  To specify a
different path to ec2.ini, define the EC2_INI_PATH environment variable:

    export EC2_INI_PATH=/path/to/my_ec2.ini

If you're using eucalyptus you need to set the above variables and
you need to define:

    export EC2_URL=http://hostname_of_your_cc:port/services/Eucalyptus

For more details, see: http://docs.pythonboto.org/en/latest/boto_config_tut.html

When run against a specific host, this script returns the following variables:
 - ec2_ami_launch_index
 - ec2_architecture
 - ec2_association
 - ec2_attachTime
 - ec2_attachment
 - ec2_attachmentId
 - ec2_client_token
 - ec2_deleteOnTermination
 - ec2_description
 - ec2_deviceIndex
 - ec2_dns_name
 - ec2_eventsSet
 - ec2_group_name
 - ec2_hypervisor
 - ec2_id
 - ec2_image_id
 - ec2_instanceState
 - ec2_instance_type
 - ec2_ipOwnerId
 - ec2_ip_address
 - ec2_item
 - ec2_kernel
 - ec2_key_name
 - ec2_launch_time
 - ec2_monitored
 - ec2_monitoring
 - ec2_networkInterfaceId
 - ec2_ownerId
 - ec2_persistent
 - ec2_placement
 - ec2_platform
 - ec2_previous_state
 - ec2_private_dns_name
 - ec2_private_ip_address
 - ec2_publicIp
 - ec2_public_dns_name
 - ec2_ramdisk
 - ec2_reason
 - ec2_region
 - ec2_requester_id
 - ec2_root_device_name
 - ec2_root_device_type
 - ec2_security_group_ids
 - ec2_security_group_names
 - ec2_shutdown_state
 - ec2_sourceDestCheck
 - ec2_spot_instance_request_id
 - ec2_state
 - ec2_state_code
 - ec2_state_reason
 - ec2_status
 - ec2_subnet_id
 - ec2_tenancy
 - ec2_virtualization_type
 - ec2_vpc_id

These variables are pulled out of a boto.ec2.instance object. There is a lack of
consistency with variable spellings (camelCase and underscores) since this
just loops through all variables the object exposes. It is preferred to use the
ones with underscores when multiple exist.

In addition, if an instance has AWS Tags associated with it, each tag is a new
variable named:
 - ec2_tag_[Key] = [Value]

Security groups are comma-separated in 'ec2_security_group_ids' and
'ec2_security_group_names'.
'''

# (c) 2012, Peter Sankauskas
#
# This file is part of Ansible,
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

######################################################################

import sys
import os
import argparse
import re
from time import time
import boto
from boto import ec2
from boto import rds
from boto import elasticache
from boto import route53
import six

from six.moves import configparser
from collections import defaultdict

try:
    import json
except ImportError:
    import simplejson as json


class Ec2Inventory(object):
    def _empty_inventory(self):
        return {"_meta" : {"hostvars" : {}}}

    def __init__(self):
        ''' Main execution path '''

        # Inventory grouped by instance IDs, tags, security groups, regions,
        # and availability zones
        self.inventory = self._empty_inventory()

        # Index of hostname (address) to instance ID
        self.index = {}

        # Read settings and parse CLI arguments
        self.read_settings()
        self.parse_cli_args()

        # Cache
        if self.args.refresh_cache:
            self.do_api_calls_update_cache()
        elif not self.is_cache_valid():
            self.do_api_calls_update_cache()

        # Data to print
        if self.args.host:
            data_to_print = self.get_host_info()

        elif self.args.list:
            # Display list of instances for inventory
            if self.inventory == self._empty_inventory():
                data_to_print = self.get_inventory_from_cache()
            else:
                data_to_print = self.json_format_dict(self.inventory, True)

        print(data_to_print)


    def is_cache_valid(self):
        ''' Determines if the cache files have expired, or if it is still valid '''

        if os.path.isfile(self.cache_path_cache):
            mod_time = os.path.getmtime(self.cache_path_cache)
            current_time = time()
            if (mod_time + self.cache_max_age) > current_time:
                if os.path.isfile(self.cache_path_index):
                    return True

        return False


    def read_settings(self):
        ''' Reads the settings from the ec2.ini file '''
        if six.PY2:
            config = configparser.SafeConfigParser()
        else:
            config = configparser.ConfigParser()
        ec2_default_ini_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'ec2.ini')
        ec2_ini_path = os.path.expanduser(os.path.expandvars(os.environ.get('EC2_INI_PATH', ec2_default_ini_path)))
        config.read(ec2_ini_path)

        # is eucalyptus?
        self.eucalyptus_host = None
        self.eucalyptus = False
        if config.has_option('ec2', 'eucalyptus'):
            self.eucalyptus = config.getboolean('ec2', 'eucalyptus')
        if self.eucalyptus and config.has_option('ec2', 'eucalyptus_host'):
            self.eucalyptus_host = config.get('ec2', 'eucalyptus_host')

        # Regions
        self.regions = []
        configRegions = config.get('ec2', 'regions')
        configRegions_exclude = config.get('ec2', 'regions_exclude')
        if (configRegions == 'all'):
            if self.eucalyptus_host:
                self.regions.append(boto.connect_euca(host=self.eucalyptus_host).region.name)
            else:
                for regionInfo in ec2.regions():
                    if regionInfo.name not in configRegions_exclude:
                        self.regions.append(regionInfo.name)
        else:
            self.regions = configRegions.split(",")

        # Destination addresses
        self.destination_variable = config.get('ec2', 'destination_variable')
        self.vpc_destination_variable = config.get('ec2', 'vpc_destination_variable')

        # Route53
        self.route53_enabled = config.getboolean('ec2', 'route53')
        self.route53_excluded_zones = []
        if config.has_option('ec2', 'route53_excluded_zones'):
            self.route53_excluded_zones.extend(
                config.get('ec2', 'route53_excluded_zones', '').split(','))

        # Include RDS instances?
        self.rds_enabled = True
        if config.has_option('ec2', 'rds'):
            self.rds_enabled = config.getboolean('ec2', 'rds')

        # Include ElastiCache instances?
        self.elasticache_enabled = True
        if config.has_option('ec2', 'elasticache'):
            self.elasticache_enabled = config.getboolean('ec2', 'elasticache')

        # Return all EC2 instances?
        if config.has_option('ec2', 'all_instances'):
            self.all_instances = config.getboolean('ec2', 'all_instances')
        else:
            self.all_instances = False

        # Instance states to be gathered in inventory. Default is 'running'.
        # Setting 'all_instances' to 'yes' overrides this option.
        ec2_valid_instance_states = [
            'pending',
            'running',
            'shutting-down',
            'terminated',
            'stopping',
            'stopped'
        ]
        self.ec2_instance_states = []
        if self.all_instances:
            self.ec2_instance_states = ec2_valid_instance_states
        elif config.has_option('ec2', 'instance_states'):
          for instance_state in config.get('ec2', 'instance_states').split(','):
            instance_state = instance_state.strip()
            if instance_state not in ec2_valid_instance_states:
              continue
            self.ec2_instance_states.append(instance_state)
        else:
          self.ec2_instance_states = ['running']

        # Return all RDS instances? (if RDS is enabled)
        if config.has_option('ec2', 'all_rds_instances') and self.rds_enabled:
            self.all_rds_instances = config.getboolean('ec2', 'all_rds_instances')
        else:
            self.all_rds_instances = False

        # Return all ElastiCache replication groups? (if ElastiCache is enabled)
        if config.has_option('ec2', 'all_elasticache_replication_groups') and self.elasticache_enabled:
            self.all_elasticache_replication_groups = config.getboolean('ec2', 'all_elasticache_replication_groups')
        else:
            self.all_elasticache_replication_groups = False

        # Return all ElastiCache clusters? (if ElastiCache is enabled)
        if config.has_option('ec2', 'all_elasticache_clusters') and self.elasticache_enabled:
            self.all_elasticache_clusters = config.getboolean('ec2', 'all_elasticache_clusters')
        else:
            self.all_elasticache_clusters = False

        # Return all ElastiCache nodes? (if ElastiCache is enabled)
        if config.has_option('ec2', 'all_elasticache_nodes') and self.elasticache_enabled:
            self.all_elasticache_nodes = config.getboolean('ec2', 'all_elasticache_nodes')
        else:
            self.all_elasticache_nodes = False

        # Cache related
        cache_dir = os.path.expanduser(config.get('ec2', 'cache_path'))
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)

        self.cache_path_cache = cache_dir + "/ansible-ec2.cache"
        self.cache_path_index = cache_dir + "/ansible-ec2.index"
        self.cache_max_age = config.getint('ec2', 'cache_max_age')

        # Configure nested groups instead of flat namespace.
        if config.has_option('ec2', 'nested_groups'):
            self.nested_groups = config.getboolean('ec2', 'nested_groups')
        else:
            self.nested_groups = False

        # Configure which groups should be created.
        group_by_options = [
            'group_by_instance_id',
            'group_by_region',
            'group_by_availability_zone',
            'group_by_ami_id',
            'group_by_instance_type',
            'group_by_key_pair',
            'group_by_vpc_id',
            'group_by_security_group',
            'group_by_tag_keys',
            'group_by_tag_none',
            'group_by_route53_names',
            'group_by_rds_engine',
            'group_by_rds_parameter_group',
            'group_by_elasticache_engine',
            'group_by_elasticache_cluster',
            'group_by_elasticache_parameter_group',
            'group_by_elasticache_replication_group',
        ]
        for option in group_by_options:
            if config.has_option('ec2', option):
                setattr(self, option, config.getboolean('ec2', option))
            else:
                setattr(self, option, True)

        # Do we need to just include hosts that match a pattern?
        try:
            pattern_include = config.get('ec2', 'pattern_include')
            if pattern_include and len(pattern_include) > 0:
                self.pattern_include = re.compile(pattern_include)
            else:
                self.pattern_include = None
        except configparser.NoOptionError as e:
            self.pattern_include = None

        # Do we need to exclude hosts that match a pattern?
        try:
            pattern_exclude = config.get('ec2', 'pattern_exclude');
            if pattern_exclude and len(pattern_exclude) > 0:
                self.pattern_exclude = re.compile(pattern_exclude)
            else:
                self.pattern_exclude = None
        except configparser.NoOptionError as e:
            self.pattern_exclude = None

        # Instance filters (see boto and EC2 API docs). Ignore invalid filters.
        self.ec2_instance_filters = defaultdict(list)
        if config.has_option('ec2', 'instance_filters'):
            for instance_filter in config.get('ec2', 'instance_filters', '').split(','):
                instance_filter = instance_filter.strip()
                if not instance_filter or '=' not in instance_filter:
                    continue
                filter_key, filter_value = [x.strip() for x in instance_filter.split('=', 1)]
                if not filter_key:
                    continue
                self.ec2_instance_filters[filter_key].append(filter_value)

    def parse_cli_args(self):
        ''' Command line argument processing '''

        parser = argparse.ArgumentParser(description='Produce an Ansible Inventory file based on EC2')
        parser.add_argument('--list', action='store_true', default=True,
                           help='List instances (default: True)')
        parser.add_argument('--host', action='store',
                           help='Get all the variables about a specific instance')
        parser.add_argument('--refresh-cache', action='store_true', default=False,
                           help='Force refresh of cache by making API requests to EC2 (default: False - use cache files)')
        self.args = parser.parse_args()


    def do_api_calls_update_cache(self):
        ''' Do API calls to each region, and save data in cache files '''

        if self.route53_enabled:
            self.get_route53_records()

        for region in self.regions:
            self.get_instances_by_region(region)
            if self.rds_enabled:
                self.get_rds_instances_by_region(region)
            if self.elasticache_enabled:
                self.get_elasticache_clusters_by_region(region)
                self.get_elasticache_replication_groups_by_region(region)

        self.write_to_cache(self.inventory, self.cache_path_cache)
        self.write_to_cache(self.index, self.cache_path_index)

    def connect(self, region):
        ''' create connection to api server'''
        if self.eucalyptus:
            conn = boto.connect_euca(host=self.eucalyptus_host)
            conn.APIVersion = '2010-08-31'
        else:
            conn = ec2.connect_to_region(region)
        # connect_to_region will fail "silently" by returning None if the region name is wrong or not supported
        if conn is None:
            self.fail_with_error("region name: %s likely not supported, or AWS is down.  connection to region failed." % region)
        return conn

    def get_instances_by_region(self, region):
        ''' Makes an AWS EC2 API call to the list of instances in a particular
        region '''

        try:
            conn = self.connect(region)
            reservations = []
            if self.ec2_instance_filters:
                for filter_key, filter_values in self.ec2_instance_filters.items():
                    reservations.extend(conn.get_all_instances(filters = { filter_key : filter_values }))
            else:
                reservations = conn.get_all_instances()

            for reservation in reservations:
                for instance in reservation.instances:
                    self.add_instance(instance, region)

        except boto.exception.BotoServerError as e:
            if e.error_code == 'AuthFailure':
                error = self.get_auth_error_message()
            else:
                backend = 'Eucalyptus' if self.eucalyptus else 'AWS' 
                error = "Error connecting to %s backend.\n%s" % (backend, e.message)
            self.fail_with_error(error, 'getting EC2 instances')

    def get_rds_instances_by_region(self, region):
        ''' Makes an AWS API call to the list of RDS instances in a particular
        region '''

        try:
            conn = rds.connect_to_region(region)
            if conn:
                instances = conn.get_all_dbinstances()
                for instance in instances:
                    self.add_rds_instance(instance, region)
        except boto.exception.BotoServerError as e:
            error = e.reason

            if e.error_code == 'AuthFailure':
                error = self.get_auth_error_message()
            if not e.reason == "Forbidden":
                error = "Looks like AWS RDS is down:\n%s" % e.message
            self.fail_with_error(error, 'getting RDS instances')

    def get_elasticache_clusters_by_region(self, region):
        ''' Makes an AWS API call to the list of ElastiCache clusters (with
        nodes' info) in a particular region.'''

        # ElastiCache boto module doesn't provide a get_all_intances method,
        # that's why we need to call describe directly (it would be called by
        # the shorthand method anyway...)
        try:
            conn = elasticache.connect_to_region(region)
            if conn:
                # show_cache_node_info = True
                # because we also want nodes' information
                response = conn.describe_cache_clusters(None, None, None, True)

        except boto.exception.BotoServerError as e:
            error = e.reason

            if e.error_code == 'AuthFailure':
                error = self.get_auth_error_message()
            if not e.reason == "Forbidden":
                error = "Looks like AWS ElastiCache is down:\n%s" % e.message
            self.fail_with_error(error, 'getting ElastiCache clusters')

        try:
            # Boto also doesn't provide wrapper classes to CacheClusters or
            # CacheNodes. Because of that wo can't make use of the get_list
            # method in the AWSQueryConnection. Let's do the work manually
            clusters = response['DescribeCacheClustersResponse']['DescribeCacheClustersResult']['CacheClusters']

        except KeyError as e:
            error = "ElastiCache query to AWS failed (unexpected format)."
            self.fail_with_error(error, 'getting ElastiCache clusters')

        for cluster in clusters:
            self.add_elasticache_cluster(cluster, region)

    def get_elasticache_replication_groups_by_region(self, region):
        ''' Makes an AWS API call to the list of ElastiCache replication groups
        in a particular region.'''

        # ElastiCache boto module doesn't provide a get_all_intances method,
        # that's why we need to call describe directly (it would be called by
        # the shorthand method anyway...)
        try:
            conn = elasticache.connect_to_region(region)
            if conn:
                response = conn.describe_replication_groups()

        except boto.exception.BotoServerError as e:
            error = e.reason

            if e.error_code == 'AuthFailure':
                error = self.get_auth_error_message()
            if not e.reason == "Forbidden":
                error = "Looks like AWS ElastiCache [Replication Groups] is down:\n%s" % e.message
            self.fail_with_error(error, 'getting ElastiCache clusters')

        try:
            # Boto also doesn't provide wrapper classes to ReplicationGroups
            # Because of that wo can't make use of the get_list method in the
            # AWSQueryConnection. Let's do the work manually
            replication_groups = response['DescribeReplicationGroupsResponse']['DescribeReplicationGroupsResult']['ReplicationGroups']

        except KeyError as e:
            error = "ElastiCache [Replication Groups] query to AWS failed (unexpected format)."
            self.fail_with_error(error, 'getting ElastiCache clusters')

        for replication_group in replication_groups:
            self.add_elasticache_replication_group(replication_group, region)

    def get_auth_error_message(self):
        ''' create an informative error message if there is an issue authenticating'''
        errors = ["Authentication error retrieving ec2 inventory."]
        if None in [os.environ.get('AWS_ACCESS_KEY_ID'), os.environ.get('AWS_SECRET_ACCESS_KEY')]:
            errors.append(' - No AWS_ACCESS_KEY_ID or AWS_SECRET_ACCESS_KEY environment vars found')
        else:
            errors.append(' - AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment vars found but may not be correct')

        boto_paths = ['/etc/boto.cfg', '~/.boto', '~/.aws/credentials']
        boto_config_found = list(p for p in boto_paths if os.path.isfile(os.path.expanduser(p)))
        if len(boto_config_found) > 0:
            errors.append(" - Boto configs found at '%s', but the credentials contained may not be correct" % ', '.join(boto_config_found))
        else:
            errors.append(" - No Boto config found at any expected location '%s'" % ', '.join(boto_paths))

        return '\n'.join(errors)

    def fail_with_error(self, err_msg, err_operation=None):
        '''log an error to std err for ansible-playbook to consume and exit'''
        if err_operation:
            err_msg = 'ERROR: "{err_msg}", while: {err_operation}'.format(
                err_msg=err_msg, err_operation=err_operation)
        sys.stderr.write(err_msg)
        sys.exit(1)

    def get_instance(self, region, instance_id):
        conn = self.connect(region)

        reservations = conn.get_all_instances([instance_id])
        for reservation in reservations:
            for instance in reservation.instances:
                return instance

    def add_instance(self, instance, region):
        ''' Adds an instance to the inventory and index, as long as it is
        addressable '''

        # Only return instances with desired instance states
        if instance.state not in self.ec2_instance_states:
            return

        # Select the best destination address
        if instance.subnet_id:
            dest = getattr(instance, self.vpc_destination_variable, None)
            if dest is None:
                dest = getattr(instance, 'tags').get(self.vpc_destination_variable, None)
        else:
            dest = getattr(instance, self.destination_variable, None)
            if dest is None:
                dest = getattr(instance, 'tags').get(self.destination_variable, None)

        if not dest:
            # Skip instances we cannot address (e.g. private VPC subnet)
            return

        # if we only want to include hosts that match a pattern, skip those that don't
        if self.pattern_include and not self.pattern_include.match(dest):
            return

        # if we need to exclude hosts that match a pattern, skip those
        if self.pattern_exclude and self.pattern_exclude.match(dest):
            return

        # Add to index
        self.index[dest] = [region, instance.id]

        # Inventory: Group by instance ID (always a group of 1)
        if self.group_by_instance_id:
            self.inventory[instance.id] = [dest]
            if self.nested_groups:
                self.push_group(self.inventory, 'instances', instance.id)

        # Inventory: Group by region
        if self.group_by_region:
            self.push(self.inventory, region, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'regions', region)

        # Inventory: Group by availability zone
        if self.group_by_availability_zone:
            self.push(self.inventory, instance.placement, dest)
            if self.nested_groups:
                if self.group_by_region:
                    self.push_group(self.inventory, region, instance.placement)
                self.push_group(self.inventory, 'zones', instance.placement)

        # Inventory: Group by Amazon Machine Image (AMI) ID
        if self.group_by_ami_id:
            ami_id = self.to_safe(instance.image_id)
            self.push(self.inventory, ami_id, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'images', ami_id)

        # Inventory: Group by instance type
        if self.group_by_instance_type:
            type_name = self.to_safe('type_' + instance.instance_type)
            self.push(self.inventory, type_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'types', type_name)

        # Inventory: Group by key pair
        if self.group_by_key_pair and instance.key_name:
            key_name = self.to_safe('key_' + instance.key_name)
            self.push(self.inventory, key_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'keys', key_name)

        # Inventory: Group by VPC
        if self.group_by_vpc_id and instance.vpc_id:
            vpc_id_name = self.to_safe('vpc_id_' + instance.vpc_id)
            self.push(self.inventory, vpc_id_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'vpcs', vpc_id_name)

        # Inventory: Group by security group
        if self.group_by_security_group:
            try:
                for group in instance.groups:
                    key = self.to_safe("security_group_" + group.name)
                    self.push(self.inventory, key, dest)
                    if self.nested_groups:
                        self.push_group(self.inventory, 'security_groups', key)
            except AttributeError:
                self.fail_with_error('\n'.join(['Package boto seems a bit older.', 
                                            'Please upgrade boto >= 2.3.0.']))

        # Inventory: Group by tag keys
        if self.group_by_tag_keys:
            for k, v in instance.tags.items():
                if v:
                    key = self.to_safe("tag_" + k + "=" + v)
                else:
                    key = self.to_safe("tag_" + k)
                self.push(self.inventory, key, dest)
                if self.nested_groups:
                    self.push_group(self.inventory, 'tags', self.to_safe("tag_" + k))
                    self.push_group(self.inventory, self.to_safe("tag_" + k), key)

        # Inventory: Group by Route53 domain names if enabled
        if self.route53_enabled and self.group_by_route53_names:
            route53_names = self.get_instance_route53_names(instance)
            for name in route53_names:
                self.push(self.inventory, name, dest)
                if self.nested_groups:
                    self.push_group(self.inventory, 'route53', name)

        # Global Tag: instances without tags
        if self.group_by_tag_none and len(instance.tags) == 0:
            self.push(self.inventory, 'tag_none', dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'tags', 'tag_none')

        # Global Tag: tag all EC2 instances
        self.push(self.inventory, 'ec2', dest)

        self.inventory["_meta"]["hostvars"][dest] = self.get_host_info_dict_from_instance(instance)


    def add_rds_instance(self, instance, region):
        ''' Adds an RDS instance to the inventory and index, as long as it is
        addressable '''

        # Only want available instances unless all_rds_instances is True
        if not self.all_rds_instances and instance.status != 'available':
            return

        # Select the best destination address
        dest = instance.endpoint[0]

        if not dest:
            # Skip instances we cannot address (e.g. private VPC subnet)
            return

        # Add to index
        self.index[dest] = [region, instance.id]

        # Inventory: Group by instance ID (always a group of 1)
        if self.group_by_instance_id:
            self.inventory[instance.id] = [dest]
            if self.nested_groups:
                self.push_group(self.inventory, 'instances', instance.id)

        # Inventory: Group by region
        if self.group_by_region:
            self.push(self.inventory, region, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'regions', region)

        # Inventory: Group by availability zone
        if self.group_by_availability_zone:
            self.push(self.inventory, instance.availability_zone, dest)
            if self.nested_groups:
                if self.group_by_region:
                    self.push_group(self.inventory, region, instance.availability_zone)
                self.push_group(self.inventory, 'zones', instance.availability_zone)

        # Inventory: Group by instance type
        if self.group_by_instance_type:
            type_name = self.to_safe('type_' + instance.instance_class)
            self.push(self.inventory, type_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'types', type_name)

        # Inventory: Group by VPC
        if self.group_by_vpc_id and instance.subnet_group and instance.subnet_group.vpc_id:
            vpc_id_name = self.to_safe('vpc_id_' + instance.subnet_group.vpc_id)
            self.push(self.inventory, vpc_id_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'vpcs', vpc_id_name)

        # Inventory: Group by security group
        if self.group_by_security_group:
            try:
                if instance.security_group:
                    key = self.to_safe("security_group_" + instance.security_group.name)
                    self.push(self.inventory, key, dest)
                    if self.nested_groups:
                        self.push_group(self.inventory, 'security_groups', key)

            except AttributeError:
                self.fail_with_error('\n'.join(['Package boto seems a bit older.', 
                                            'Please upgrade boto >= 2.3.0.']))


        # Inventory: Group by engine
        if self.group_by_rds_engine:
            self.push(self.inventory, self.to_safe("rds_" + instance.engine), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'rds_engines', self.to_safe("rds_" + instance.engine))

        # Inventory: Group by parameter group
        if self.group_by_rds_parameter_group:
            self.push(self.inventory, self.to_safe("rds_parameter_group_" + instance.parameter_group.name), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'rds_parameter_groups', self.to_safe("rds_parameter_group_" + instance.parameter_group.name))

        # Global Tag: all RDS instances
        self.push(self.inventory, 'rds', dest)

        self.inventory["_meta"]["hostvars"][dest] = self.get_host_info_dict_from_instance(instance)

    def add_elasticache_cluster(self, cluster, region):
        ''' Adds an ElastiCache cluster to the inventory and index, as long as
        it's nodes are addressable '''

        # Only want available clusters unless all_elasticache_clusters is True
        if not self.all_elasticache_clusters and cluster['CacheClusterStatus'] != 'available':
            return

        # Select the best destination address
        if 'ConfigurationEndpoint' in cluster and cluster['ConfigurationEndpoint']:
            # Memcached cluster
            dest = cluster['ConfigurationEndpoint']['Address']
            is_redis = False
        else:
            # Redis sigle node cluster
            # Because all Redis clusters are single nodes, we'll merge the
            # info from the cluster with info about the node
            dest = cluster['CacheNodes'][0]['Endpoint']['Address']
            is_redis = True

        if not dest:
            # Skip clusters we cannot address (e.g. private VPC subnet)
            return

        # Add to index
        self.index[dest] = [region, cluster['CacheClusterId']]

        # Inventory: Group by instance ID (always a group of 1)
        if self.group_by_instance_id:
            self.inventory[cluster['CacheClusterId']] = [dest]
            if self.nested_groups:
                self.push_group(self.inventory, 'instances', cluster['CacheClusterId'])

        # Inventory: Group by region
        if self.group_by_region and not is_redis:
            self.push(self.inventory, region, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'regions', region)

        # Inventory: Group by availability zone
        if self.group_by_availability_zone and not is_redis:
            self.push(self.inventory, cluster['PreferredAvailabilityZone'], dest)
            if self.nested_groups:
                if self.group_by_region:
                    self.push_group(self.inventory, region, cluster['PreferredAvailabilityZone'])
                self.push_group(self.inventory, 'zones', cluster['PreferredAvailabilityZone'])

        # Inventory: Group by node type
        if self.group_by_instance_type and not is_redis:
            type_name = self.to_safe('type_' + cluster['CacheNodeType'])
            self.push(self.inventory, type_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'types', type_name)

        # Inventory: Group by VPC (information not available in the current
        # AWS API version for ElastiCache)

        # Inventory: Group by security group
        if self.group_by_security_group and not is_redis:

            # Check for the existence of the 'SecurityGroups' key and also if
            # this key has some value. When the cluster is not placed in a SG
            # the query can return None here and cause an error.
            if 'SecurityGroups' in cluster and cluster['SecurityGroups'] is not None:
                for security_group in cluster['SecurityGroups']:
                    key = self.to_safe("security_group_" + security_group['SecurityGroupId'])
                    self.push(self.inventory, key, dest)
                    if self.nested_groups:
                        self.push_group(self.inventory, 'security_groups', key)

        # Inventory: Group by engine
        if self.group_by_elasticache_engine and not is_redis:
            self.push(self.inventory, self.to_safe("elasticache_" + cluster['Engine']), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'elasticache_engines', self.to_safe(cluster['Engine']))

        # Inventory: Group by parameter group
        if self.group_by_elasticache_parameter_group:
            self.push(self.inventory, self.to_safe("elasticache_parameter_group_" + cluster['CacheParameterGroup']['CacheParameterGroupName']), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'elasticache_parameter_groups', self.to_safe(cluster['CacheParameterGroup']['CacheParameterGroupName']))

        # Inventory: Group by replication group
        if self.group_by_elasticache_replication_group and 'ReplicationGroupId' in cluster and cluster['ReplicationGroupId']:
            self.push(self.inventory, self.to_safe("elasticache_replication_group_" + cluster['ReplicationGroupId']), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'elasticache_replication_groups', self.to_safe(cluster['ReplicationGroupId']))

        # Global Tag: all ElastiCache clusters
        self.push(self.inventory, 'elasticache_clusters', cluster['CacheClusterId'])

        host_info = self.get_host_info_dict_from_describe_dict(cluster)

        self.inventory["_meta"]["hostvars"][dest] = host_info

        # Add the nodes
        for node in cluster['CacheNodes']:
            self.add_elasticache_node(node, cluster, region)

    def add_elasticache_node(self, node, cluster, region):
        ''' Adds an ElastiCache node to the inventory and index, as long as
        it is addressable '''

        # Only want available nodes unless all_elasticache_nodes is True
        if not self.all_elasticache_nodes and node['CacheNodeStatus'] != 'available':
            return

        # Select the best destination address
        dest = node['Endpoint']['Address']

        if not dest:
            # Skip nodes we cannot address (e.g. private VPC subnet)
            return

        node_id = self.to_safe(cluster['CacheClusterId'] + '_' + node['CacheNodeId'])

        # Add to index
        self.index[dest] = [region, node_id]

        # Inventory: Group by node ID (always a group of 1)
        if self.group_by_instance_id:
            self.inventory[node_id] = [dest]
            if self.nested_groups:
                self.push_group(self.inventory, 'instances', node_id)

        # Inventory: Group by region
        if self.group_by_region:
            self.push(self.inventory, region, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'regions', region)

        # Inventory: Group by availability zone
        if self.group_by_availability_zone:
            self.push(self.inventory, cluster['PreferredAvailabilityZone'], dest)
            if self.nested_groups:
                if self.group_by_region:
                    self.push_group(self.inventory, region, cluster['PreferredAvailabilityZone'])
                self.push_group(self.inventory, 'zones', cluster['PreferredAvailabilityZone'])

        # Inventory: Group by node type
        if self.group_by_instance_type:
            type_name = self.to_safe('type_' + cluster['CacheNodeType'])
            self.push(self.inventory, type_name, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'types', type_name)

        # Inventory: Group by VPC (information not available in the current
        # AWS API version for ElastiCache)

        # Inventory: Group by security group
        if self.group_by_security_group:

            # Check for the existence of the 'SecurityGroups' key and also if
            # this key has some value. When the cluster is not placed in a SG
            # the query can return None here and cause an error.
            if 'SecurityGroups' in cluster and cluster['SecurityGroups'] is not None:
                for security_group in cluster['SecurityGroups']:
                    key = self.to_safe("security_group_" + security_group['SecurityGroupId'])
                    self.push(self.inventory, key, dest)
                    if self.nested_groups:
                        self.push_group(self.inventory, 'security_groups', key)

        # Inventory: Group by engine
        if self.group_by_elasticache_engine:
            self.push(self.inventory, self.to_safe("elasticache_" + cluster['Engine']), dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'elasticache_engines', self.to_safe("elasticache_" + cluster['Engine']))

        # Inventory: Group by parameter group (done at cluster level)

        # Inventory: Group by replication group (done at cluster level)

        # Inventory: Group by ElastiCache Cluster
        if self.group_by_elasticache_cluster:
            self.push(self.inventory, self.to_safe("elasticache_cluster_" + cluster['CacheClusterId']), dest)

        # Global Tag: all ElastiCache nodes
        self.push(self.inventory, 'elasticache_nodes', dest)

        host_info = self.get_host_info_dict_from_describe_dict(node)

        if dest in self.inventory["_meta"]["hostvars"]:
            self.inventory["_meta"]["hostvars"][dest].update(host_info)
        else:
            self.inventory["_meta"]["hostvars"][dest] = host_info

    def add_elasticache_replication_group(self, replication_group, region):
        ''' Adds an ElastiCache replication group to the inventory and index '''

        # Only want available clusters unless all_elasticache_replication_groups is True
        if not self.all_elasticache_replication_groups and replication_group['Status'] != 'available':
            return

        # Select the best destination address (PrimaryEndpoint)
        dest = replication_group['NodeGroups'][0]['PrimaryEndpoint']['Address']

        if not dest:
            # Skip clusters we cannot address (e.g. private VPC subnet)
            return

        # Add to index
        self.index[dest] = [region, replication_group['ReplicationGroupId']]

        # Inventory: Group by ID (always a group of 1)
        if self.group_by_instance_id:
            self.inventory[replication_group['ReplicationGroupId']] = [dest]
            if self.nested_groups:
                self.push_group(self.inventory, 'instances', replication_group['ReplicationGroupId'])

        # Inventory: Group by region
        if self.group_by_region:
            self.push(self.inventory, region, dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'regions', region)

        # Inventory: Group by availability zone (doesn't apply to replication groups)

        # Inventory: Group by node type (doesn't apply to replication groups)

        # Inventory: Group by VPC (information not available in the current
        # AWS API version for replication groups

        # Inventory: Group by security group (doesn't apply to replication groups)
        # Check this value in cluster level

        # Inventory: Group by engine (replication groups are always Redis)
        if self.group_by_elasticache_engine:
            self.push(self.inventory, 'elasticache_redis', dest)
            if self.nested_groups:
                self.push_group(self.inventory, 'elasticache_engines', 'redis')

        # Global Tag: all ElastiCache clusters
        self.push(self.inventory, 'elasticache_replication_groups', replication_group['ReplicationGroupId'])

        host_info = self.get_host_info_dict_from_describe_dict(replication_group)

        self.inventory["_meta"]["hostvars"][dest] = host_info

    def get_route53_records(self):
        ''' Get and store the map of resource records to domain names that
        point to them. '''

        r53_conn = route53.Route53Connection()
        all_zones = r53_conn.get_zones()

        route53_zones = [ zone for zone in all_zones if zone.name[:-1]
                          not in self.route53_excluded_zones ]

        self.route53_records = {}

        for zone in route53_zones:
            rrsets = r53_conn.get_all_rrsets(zone.id)

            for record_set in rrsets:
                record_name = record_set.name

                if record_name.endswith('.'):
                    record_name = record_name[:-1]

                for resource in record_set.resource_records:
                    self.route53_records.setdefault(resource, set())
                    self.route53_records[resource].add(record_name)


    def get_instance_route53_names(self, instance):
        ''' Check if an instance is referenced in the records we have from
        Route53. If it is, return the list of domain names pointing to said
        instance. If nothing points to it, return an empty list. '''

        instance_attributes = [ 'public_dns_name', 'private_dns_name',
                                'ip_address', 'private_ip_address' ]

        name_list = set()

        for attrib in instance_attributes:
            try:
                value = getattr(instance, attrib)
            except AttributeError:
                continue

            if value in self.route53_records:
                name_list.update(self.route53_records[value])

        return list(name_list)

    def get_host_info_dict_from_instance(self, instance):
        instance_vars = {}
        for key in vars(instance):
            value = getattr(instance, key)
            key = self.to_safe('ec2_' + key)

            # Handle complex types
            # state/previous_state changed to properties in boto in https://github.com/boto/boto/commit/a23c379837f698212252720d2af8dec0325c9518
            if key == 'ec2__state':
                instance_vars['ec2_state'] = instance.state or ''
                instance_vars['ec2_state_code'] = instance.state_code
            elif key == 'ec2__previous_state':
                instance_vars['ec2_previous_state'] = instance.previous_state or ''
                instance_vars['ec2_previous_state_code'] = instance.previous_state_code
            elif type(value) in [int, bool]:
                instance_vars[key] = value
            elif isinstance(value, six.string_types):
                instance_vars[key] = value.strip()
            elif type(value) == type(None):
                instance_vars[key] = ''
            elif key == 'ec2_region':
                instance_vars[key] = value.name
            elif key == 'ec2__placement':
                instance_vars['ec2_placement'] = value.zone
            elif key == 'ec2_tags':
                for k, v in value.items():
                    key = self.to_safe('ec2_tag_' + k)
                    instance_vars[key] = v
            elif key == 'ec2_groups':
                group_ids = []
                group_names = []
                for group in value:
                    group_ids.append(group.id)
                    group_names.append(group.name)
                instance_vars["ec2_security_group_ids"] = ','.join([str(i) for i in group_ids])
                instance_vars["ec2_security_group_names"] = ','.join([str(i) for i in group_names])
            else:
                pass
                # TODO Product codes if someone finds them useful
                #print key
                #print type(value)
                #print value

        return instance_vars

    def get_host_info_dict_from_describe_dict(self, describe_dict):
        ''' Parses the dictionary returned by the API call into a flat list
            of parameters. This method should be used only when 'describe' is
            used directly because Boto doesn't provide specific classes. '''

        # I really don't agree with prefixing everything with 'ec2'
        # because EC2, RDS and ElastiCache are different services.
        # I'm just following the pattern used until now to not break any
        # compatibility.

        host_info = {}
        for key in describe_dict:
            value = describe_dict[key]
            key = self.to_safe('ec2_' + self.uncammelize(key))

            # Handle complex types

            # Target: Memcached Cache Clusters
            if key == 'ec2_configuration_endpoint' and value:
                host_info['ec2_configuration_endpoint_address'] = value['Address']
                host_info['ec2_configuration_endpoint_port'] = value['Port']

            # Target: Cache Nodes and Redis Cache Clusters (single node)
            if key == 'ec2_endpoint' and value:
                host_info['ec2_endpoint_address'] = value['Address']
                host_info['ec2_endpoint_port'] = value['Port']

            # Target: Redis Replication Groups
            if key == 'ec2_node_groups' and value:
                host_info['ec2_endpoint_address'] = value[0]['PrimaryEndpoint']['Address']
                host_info['ec2_endpoint_port'] = value[0]['PrimaryEndpoint']['Port']
                replica_count = 0
                for node in value[0]['NodeGroupMembers']:
                    if node['CurrentRole'] == 'primary':
                        host_info['ec2_primary_cluster_address'] = node['ReadEndpoint']['Address']
                        host_info['ec2_primary_cluster_port'] = node['ReadEndpoint']['Port']
                        host_info['ec2_primary_cluster_id'] = node['CacheClusterId']
                    elif node['CurrentRole'] == 'replica':
                        host_info['ec2_replica_cluster_address_'+ str(replica_count)] = node['ReadEndpoint']['Address']
                        host_info['ec2_replica_cluster_port_'+ str(replica_count)] = node['ReadEndpoint']['Port']
                        host_info['ec2_replica_cluster_id_'+ str(replica_count)] = node['CacheClusterId']
                        replica_count += 1

            # Target: Redis Replication Groups
            if key == 'ec2_member_clusters' and value:
                host_info['ec2_member_clusters'] = ','.join([str(i) for i in value])

            # Target: All Cache Clusters
            elif key == 'ec2_cache_parameter_group':
                host_info["ec2_cache_node_ids_to_reboot"] = ','.join([str(i) for i in value['CacheNodeIdsToReboot']])
                host_info['ec2_cache_parameter_group_name'] = value['CacheParameterGroupName']
                host_info['ec2_cache_parameter_apply_status'] = value['ParameterApplyStatus']

            # Target: Almost everything
            elif key == 'ec2_security_groups':

                # Skip if SecurityGroups is None
                # (it is possible to have the key defined but no value in it).
                if value is not None:
                    sg_ids = []
                    for sg in value:
                        sg_ids.append(sg['SecurityGroupId'])
                    host_info["ec2_security_group_ids"] = ','.join([str(i) for i in sg_ids])

            # Target: Everything
            # Preserve booleans and integers
            elif type(value) in [int, bool]:
                host_info[key] = value

            # Target: Everything
            # Sanitize string values
            elif isinstance(value, six.string_types):
                host_info[key] = value.strip()

            # Target: Everything
            # Replace None by an empty string
            elif type(value) == type(None):
                host_info[key] = ''

            else:
                # Remove non-processed complex types
                pass

        return host_info

    def get_host_info(self):
        ''' Get variables about a specific host '''

        if len(self.index) == 0:
            # Need to load index from cache
            self.load_index_from_cache()

        if not self.args.host in self.index:
            # try updating the cache
            self.do_api_calls_update_cache()
            if not self.args.host in self.index:
                # host might not exist anymore
                return self.json_format_dict({}, True)

        (region, instance_id) = self.index[self.args.host]

        instance = self.get_instance(region, instance_id)
        return self.json_format_dict(self.get_host_info_dict_from_instance(instance), True)

    def push(self, my_dict, key, element):
        ''' Push an element onto an array that may not have been defined in
        the dict '''
        group_info = my_dict.setdefault(key, [])
        if isinstance(group_info, dict):
            host_list = group_info.setdefault('hosts', [])
            host_list.append(element)
        else:
            group_info.append(element)

    def push_group(self, my_dict, key, element):
        ''' Push a group as a child of another group. '''
        parent_group = my_dict.setdefault(key, {})
        if not isinstance(parent_group, dict):
            parent_group = my_dict[key] = {'hosts': parent_group}
        child_groups = parent_group.setdefault('children', [])
        if element not in child_groups:
            child_groups.append(element)

    def get_inventory_from_cache(self):
        ''' Reads the inventory from the cache file and returns it as a JSON
        object '''

        cache = open(self.cache_path_cache, 'r')
        json_inventory = cache.read()
        return json_inventory


    def load_index_from_cache(self):
        ''' Reads the index from the cache file sets self.index '''

        cache = open(self.cache_path_index, 'r')
        json_index = cache.read()
        self.index = json.loads(json_index)


    def write_to_cache(self, data, filename):
        ''' Writes data in JSON format to a file '''

        json_data = self.json_format_dict(data, True)
        cache = open(filename, 'w')
        cache.write(json_data)
        cache.close()

    def uncammelize(self, key):
        temp = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', key)
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', temp).lower()

    def to_safe(self, word):
        ''' Converts 'bad' characters in a string to underscores so they can be
        used as Ansible groups '''

        return re.sub("[^A-Za-z0-9\_]", "_", word)

    def json_format_dict(self, data, pretty=False):
        ''' Converts a dict to a JSON object and dumps it as a formatted
        string '''

        if pretty:
            return json.dumps(data, sort_keys=True, indent=2)
        else:
            return json.dumps(data)


# Run the script
Ec2Inventory()




from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict


class Migration(migration.Migration):
    _starting_version = '2.2.0'
    _ending_version = '2.3.0'

    def run(self):

        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            logging.debug('Inventory Path: {}'.format(inventory_path))
            self.delete('{}/config/local/local-config-init'.format(inventory_path))
            self.delete('{}/terraform/Makefile'.format(inventory_path))



# This function will take an irregular list composed of lists 
# and flatten it

from compiler.ast import flatten

class FilterModule (object):
    def filters(self):
        return {
            "flatten": flatten
        }



from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict


class Migration(migration.Migration):
    _starting_version = '2.4.1'
    _ending_version = '2.4.2'

    def run(self):
        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            self.delete("{}/terraform/Makefile".format(inventory_path))

        self.delete('Makefile')



import logging
import os
import traceback
import jinja2
import string

import oyaml as yaml
from Crypto.PublicKey import RSA
from stat import *
from collections import OrderedDict


def render_template(template_name, template_path, target, context, delete_template=True, overwrite=False):
    """
    Helper function to write out DRY up templating. Accepts template name (string),
    path (string), target directory (string), context (dictionary) and delete_template (boolean)
    Default behavior is to use the key of the dictionary as the template variable names, replace
    them with the value in the tempalate and delete the template if delete_template is
    True
    """

    logging.info("Writing {}".format(target))
    logging.debug("Template Context: {}".format(context))
    logging.debug("Overwrite is {}".format(overwrite))
    if os.path.isfile(target) and overwrite is not True:
        logging.warn("Cowardly refusing to overwrite existing file {}".format(target))
        return False

    logging.debug("Attempting to write {} from template {}/{}".format(target, template_path, template_name))

    template_path = os.path.normpath(template_path)
    template_name = os.path.normpath(template_name)
    template_permissions = os.stat(template_path + '/' + template_name).st_mode

    with open(target, 'w+') as vars_file:
        try:
            template = jinja2.Environment(loader=jinja2.FileSystemLoader(template_path)).get_template(template_name)
            vars_file.write(template.render(context))
        except Exception, e:
            logging.error("Error writing {}. {}".format(target, traceback.print_exc(e)))
            return False

    os.chmod(target, template_permissions)

    if delete_template:
        logging.debug("Removing {}/{}".format(template_path, template_name))
        os.remove("{}/{}".format(template_path, template_name))


def write_yaml_file(filename, d, overwrite=False):
    """ Accepts  filepath,  dictionary. Writes dictionary in yaml to file path, recursively creating path if necessary """
    if not os.path.exists(os.path.dirname(filename)) and overwrite is False:
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise
    logging.debug("Writing yaml file {}".format(filename))
    logging.debug(d)
    with open(filename, 'w+') as f:
        yaml.safe_dump(d, f, default_flow_style=False)


def create_rsa_key(name, path, bits=2048):
    """ creates an ssh key pair. Accepts name, path and bits. Name is the name of the key pair to generate at Path. Bits defaults to 2048 """

    key = RSA.generate(bits)

    private_key = "{}{}".format(path, name)
    public_key = "{}{}.pub".format(path, name)

    with open(private_key, 'w') as content_file:
        os.chmod(private_key, 0600)
        content_file.write(key.exportKey('PEM'))

    pubkey = key.publickey()
    with open(public_key, 'w') as content_file:
        content_file.write(pubkey.exportKey('OpenSSH'))


def merge_dict(d, new_data, clobber=False):
    """ accepts new_data (dict) and clobbber (boolean). Merges dictionary with dictionary 'd'. If clobber is True, overwrites value. Defaults to false """
    for key, value in new_data.items():
        if d.get(key) is None or clobber:
            logging.debug("Setting component data {}: {}".format(key, value))
            d[key] = value
    return d


def allege_aws_availability_zones(region, count):
    """
    Accepts a region (string) and count (int) and returns a list of potential aws availability zones
    It does no verification that the region is correct or that the az actually exists
    Ex: for region 'us-west-1' and count '3' it will return ['us-west-1a', 'us-west-1b', 'us-west-1c']
    """
    azs = []
    logging.info("Guessing at default AWS AZs")
    for i in range(0, count):
        azs += ["{}{}".format(region, list(string.ascii_lowercase)[i])]

    return (", ").join(azs)



from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict

import re


class Migration(migration.Migration):
    _starting_version = '2.6.0'
    _ending_version = '2.6.1'

    def run(self):
        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            logging.debug(
                'Processing Inventory Item: {}'
                .format(inventory_path)
            )

            # Remove deprecated variables from VPC TF module usage
            aws_vpc_vars_file = "{}/terraform/aws_vpc_variables.tf".format(inventory_path)
            if os.path.exists(aws_vpc_vars_file):
                aws_vpc_vars_file_content = self.get_file_content(aws_vpc_vars_file)
                aws_vpc_vars_file_content = re.sub(r'\n\s*variable "aws_access_key" {}', '', aws_vpc_vars_file_content)
                aws_vpc_vars_file_content = re.sub(r'\n\s*variable "aws_secret_key" {}', '', aws_vpc_vars_file_content)
                self.overwrite_file(aws_vpc_vars_file, aws_vpc_vars_file_content)
                logging.info('Deprecated Terraform VPC module variables removed in {}'.format(item))



# from __future__ import (absolute_import, division, print_function)
# __metaclass__ = type

import datetime
import shutil
import logging
import os
import re
import sys
import traceback
import oyaml as yaml
import boto3

from git import Repo, Git
from shutil import copytree, ignore_patterns

import component.kops as kops
import component.inventory as inventory
import component.core as core
import component.gcp as gcp
from helpers import render_template, write_yaml_file, create_rsa_key, merge_dict, allege_aws_availability_zones
from meta import __version__, __author__


class PentagonException(Exception):
    pass


class PentagonProject(object):
    from defaults import AWSPentagonDefaults as PentagonDefaults
    keys_to_sanitize = ['aws_access_key', 'aws_secret_key', 'output_file']

    def __init__(self, name, data={}):
        self._data = data
        self._name = name
        logging.debug(self._data)

        self._force = self.get_data('force')
        self._configure_project = self.get_data('configure')

        # Set this before it gets overridden by the config file
        self._outfile = self.get_data('output_file')

        # Setting local path info
        self._repository_name = os.path.expanduser("{}-infrastructure".format(name))
        self._repository_directory = "{}".format(
            self._repository_name)

        self._infrastructure_bucket = self.get_data('infrastructure_bucket', self._repository_name)

        self._private_path = "inventory/default/config/private"

    def get_data(self, name, default=None):
        """ Get argument name from click arguments, if it exists, or return default.
            Builtin .get method is inadequate because click defaults to a value of None
            which fools the .get() method """
        if self._data.get(name) is not None:
            return self._data.get(name)
        return default

    def __git_init(self):
        """ Initialize git repository in the project infrastructure path """
        Repo.init(self._repository_directory)

    def __write_config_file(self):
        """ Write sanitized yaml file of starting arguments """
        logging.info("Writing arguments to file for Posterity: {}".format(self._outfile))
        config = {}

        for key, value in self._data.items():
            if value and key not in self.keys_to_sanitize:
                config[key] = value
        config['project_name'] = self._name

        logging.debug(config)
        try:
            write_yaml_file(self._repository_directory + "/" + self._outfile, config)
        except Exception as e:
            logging.debug(traceback.format_exc(e))
            logging.error("Failed to write arguments to file")
            logging.error(e)

    def __repository_directory_exists(self):
        """ Tests if the repository directory already exists """
        logging.debug("Checking for repository {}".format(self._repository_directory))
        if os.path.isdir(self._repository_directory):
            return True
            logging.debug("Already Exists")
        logging.debug("Does not exist")
        return False

    def start(self):
        if not self.__repository_directory_exists() or self._force:
            logging.info("Copying project files...")
            self.__create_repo_core()
            self.__git_init()
            self.__write_config_file()
            with open('{}/.version'.format(self._repository_directory), 'w') as f:
                f.write(__version__)

            if self._configure_project is not False:
                self.configure_default_project()
        else:
            raise PentagonException('Project path exists. Cowardly refusing to overwrite existing project.')

    def __create_repo_core(self):
        logging.debug(self._repository_directory)
        core.Core(self._data).add('{}'.format(self._repository_directory))


class AWSPentagonProject(PentagonProject):
    # Placeholders for when there is not sensible default

    # AWS and VPC
    _aws_access_key_placeholder = '<aws-access-key>'
    _aws_secret_key_placeholder = '<aws-secret-key>'
    _aws_default_region_placeholder = '<aws-default-region>'
    _aws_availability_zone_count_placeholder = '<aws-availability-zone-count>'
    _aws_availability_zones_placeholder = '<aws-availability-zones>'

    # VPC
    _vpc_name = '<vpc_name>'
    _vpc_cidr_base = '<vpc_cidr_base>'
    _vpc_id = '<vpc_id>'

    # Working Kubernetes
    _working_kubernetes_cluster_name = '<working_kubernetes_cluster_name>'
    _working_kubernetes_dns_zone = '<working_kubernetes_dns_zone>'
    _working_kubernetes_master_aws_zone = '<working_kubernetes_master_aws_zone>'

    # Production Kubernetes
    _production_kubernetes_cluster_name = '<production_kubernetes_cluster_name>'
    _production_kubernetes_dns_zone = '<production_kubernetes_dns_zone>'
    _production_kubernetes_node_count = '<production_kubernetes_node_count>'
    _production_kubernetes_master_aws_zone = '<production_kubernetes_master_aws_zone>'


    def __init__(self, name, data={}):
        super(AWSPentagonProject, self).__init__(name, data)
        self._create_keys = self.get_data('create_keys')

        self._ssh_keys = {
            'admin_vpn_key': self.get_data('admin_vpn_key', self.PentagonDefaults.ssh['admin_vpn_key']),
            'working_kube_key': self.get_data('working_kube_key', self.PentagonDefaults.ssh['working_kube_key']),
            'production_kube_key': self.get_data('production_kube_key', self.PentagonDefaults.ssh['production_kube_key']),
            'working_private_key': self.get_data('working_private_key', self.PentagonDefaults.ssh['working_private_key']),
            'production_private_key': self.get_data('production_private_key', self.PentagonDefaults.ssh['production_private_key']),
        }

        # AWS Specific Stuff
        self._aws_access_key = self.get_data('aws_access_key', self._aws_access_key_placeholder)
        self._aws_secret_key = self.get_data('aws_secret_key', self._aws_secret_key_placeholder)
        if self.get_data('aws_default_region'):
            self._aws_default_region = self.get_data('aws_default_region')
            self._aws_availability_zone_count = int(self.get_data('aws_availability_zone_count', self.PentagonDefaults.vpc['aws_availability_zone_count']))
            self._aws_availability_zones = self.get_data('aws_availability_zones')
        else:
            self._aws_default_region = self._aws_default_region_placeholder
            self._aws_availability_zone_count = self._aws_availability_zone_count_placeholder
            self._aws_availability_zones = self._aws_availability_zones_placeholder

        # VPC information
        self._vpc_name = self.get_data('vpc_name', self.PentagonDefaults.vpc['vpc_name'])
        self._vpc_cidr_base = self.get_data('vpc_cidr_base', self.PentagonDefaults.vpc['vpc_cidr_base'])
        self._vpc_id = self.get_data('vpc_id', self._vpc_id)

        # DNS
        self._dns_zone = self.get_data('dns_zone', '{}.com'.format(self._name))

        # Kubernetes version
        self._kubernetes_version = self.get_data('kubernetes_version', self.PentagonDefaults.kubernetes['kubernetes_version'])

        # Working Kubernetes
        self._working_kubernetes_cluster_name = self.get_data('working_kubernetes_cluster_name', 'working-1.{}'.format(self._dns_zone))
        self._working_kubernetes_dns_zone = self.get_data('working_kubernetes_dns_zone', '{}'.format(self._dns_zone))

        self._working_kubernetes_node_count = self.get_data('working_kubernetes_node_count', self.PentagonDefaults.kubernetes['node_count'])
        self._working_kubernetes_master_aws_zones = self.get_data('working_kubernetes_master_aws_zones', self._aws_availability_zones)
        self._working_kubernetes_master_node_type = self.get_data('working_kubernetes_master_node_type', self.PentagonDefaults.kubernetes['master_node_type'])
        self._working_kubernetes_worker_node_type = self.get_data('working_kubernetes_worker_node_type', self.PentagonDefaults.kubernetes['worker_node_type'])
        self._working_kubernetes_v_log_level = self.get_data('working_kubernetes_v_log_level', self.PentagonDefaults.kubernetes['v_log_level'])
        self._working_kubernetes_network_cidr = self.get_data('working_kubernetes_network_cidr', self.PentagonDefaults.kubernetes['network_cidr'])
        self._working_third_octet = self.get_data('working_third_octet', self.PentagonDefaults.kubernetes['working_third_octet'])

        # Production Kubernetes
        self._production_kubernetes_cluster_name = self.get_data('production_kubernetes_cluster_name', 'production-1.{}'.format(self._dns_zone))
        self._production_kubernetes_dns_zone = self.get_data('production_kubernetes_dns_zone', '{}'.format(self._dns_zone))

        self._production_kubernetes_node_count = self.get_data('production_kubernetes_node_count', self.PentagonDefaults.kubernetes['node_count'])
        self._production_kubernetes_master_aws_zones = self.get_data('production_kubernetes_master_aws_zones', self._aws_availability_zones)
        self._production_kubernetes_master_node_type = self.get_data('production_kubernetes_master_node_type', self.PentagonDefaults.kubernetes['master_node_type'])
        self._production_kubernetes_worker_node_type = self.get_data('production_kubernetes_worker_node_type', self.PentagonDefaults.kubernetes['worker_node_type'])
        self._production_kubernetes_v_log_level = self.get_data('production_kubernetes_v_log_level', self.PentagonDefaults.kubernetes['v_log_level'])
        self._production_kubernetes_network_cidr = self.get_data('production_kubernetes_network_cidr', self.PentagonDefaults.kubernetes['network_cidr'])
        self._production_third_octet = self.get_data('production_third_octet', self.PentagonDefaults.kubernetes['production_third_octet'])

        self._vpn_ami_id = self.get_data('vpn_ami_id')

    @property
    def context(self):
        self._context = {
            'aws_secret_key': self._aws_secret_key,
            'aws_access_key': self._aws_access_key,
            'org_name': self._name,
            'vpc_name': self._vpc_name,
            'aws_default_region': self._aws_default_region,
            'aws_availability_zones': self._aws_availability_zones,
            'aws_availability_zone_count': self._aws_availability_zone_count,
            'infrastructure_bucket': self._infrastructure_bucket,
            'vpc_name': self._vpc_name,
            'vpc_cidr_base': self._vpc_cidr_base,
            'aws_availability_zones': self._aws_availability_zones,
            'aws_availability_zone_count': self._aws_availability_zone_count,
            'infrastructure_bucket': self._infrastructure_bucket,
            'vpc_name': self._vpc_name,
            'infrastructure_bucket': self._infrastructure_bucket,
            'aws_region': self._aws_default_region,
            'dns_zone': self._dns_zone,
            'vpn_ami_id': self._vpn_ami_id,
            'production_kube_key': self._ssh_keys['production_kube_key'],
            'working_kube_key': self._ssh_keys['working_kube_key'],
            'production_private_key': self._ssh_keys['production_private_key'],
            'working_private_key': self._ssh_keys['working_private_key'],
            'admin_vpn_key': self._ssh_keys['admin_vpn_key'],
            'name': 'default',
            'project_name': self._name,
            'configure_vpn': self.get_data('configure_vpn')
        }
        logging.debug(self._context)
        return self._context

    def __add_kops_working_cluster(self):
        context = {
            'cluster_name': self._working_kubernetes_cluster_name,
            'availability_zones': re.sub(" ", "", self._aws_availability_zones).split(","),
            'vpc_id': self._vpc_id,
            'ssh_key_path': "${{INFRASTRUCTURE_REPO}}/{}/{}.pub".format(self._private_path, self._ssh_keys['working_kube_key']),
            'kubernetes_version': self._kubernetes_version,
            'ig_max_size': self._working_kubernetes_node_count,
            'ig_min_size': self._working_kubernetes_node_count,
            'master_availability_zones': [zone.strip() for zone in self._working_kubernetes_master_aws_zones.split(',')],
            'master_node_type': self._working_kubernetes_master_node_type,
            'worker_node_type': self._working_kubernetes_worker_node_type,
            'cluster_dns': self._working_kubernetes_dns_zone,
            'kubernetes_v_log_level': self._working_kubernetes_v_log_level,
            'network_cidr': self._working_kubernetes_network_cidr,
            'network_cidr_base': self._vpc_cidr_base,
            'kops_state_store_bucket': self._infrastructure_bucket,
            'third_octet': self._working_third_octet
        }
        write_yaml_file("{}/inventory/default/clusters/working/vars.yml".format(self._repository_directory), context)

    def __add_kops_production_cluster(self):
        context = {
            'cluster_name': self._production_kubernetes_cluster_name,
            'availability_zones': re.sub(" ", "", self._aws_availability_zones).split(","),
            'vpc_id': self._vpc_id,
            'ssh_key_path': "${{INFRASTRUCTURE_REPO}}/{}/{}.pub".format(self._private_path, self._ssh_keys['production_kube_key']),
            'kubernetes_version': self._kubernetes_version,
            'ig_max_size': self._production_kubernetes_node_count,
            'ig_min_size': self._production_kubernetes_node_count,
            'master_availability_zones': [zone.strip() for zone in self._production_kubernetes_master_aws_zones.split(',')],
            'master_node_type': self._production_kubernetes_master_node_type,
            'worker_node_type': self._production_kubernetes_worker_node_type,
            'cluster_dns': self._production_kubernetes_dns_zone,
            'kubernetes_v_log_level': self._production_kubernetes_v_log_level,
            'network_cidr': self._production_kubernetes_network_cidr,
            'network_cidr_base': self._vpc_cidr_base,
            'kops_state_store_bucket': self._infrastructure_bucket,
            'third_octet': self._production_third_octet
        }
        write_yaml_file("{}/inventory/default/clusters/production/vars.yml".format(self._repository_directory), context)

    def configure_default_project(self):
            inventory.Inventory(self.context).add('{}/inventory/default'.format(self._repository_directory))
            self.__add_kops_working_cluster()
            self.__add_kops_production_cluster()


class GCPPentagonProject(PentagonProject):
    from defaults import GCPPentagonDefaults as PentagonDefaults

    local_defaults = {
        'working_cluster_name': 'working',
        'production_cluster_name': 'production',
    }

    def __init__(self, name, data={}):
        super(GCPPentagonProject, self).__init__(name, data)
        self.default_context = self.PentagonDefaults.kubernetes
        if type(self.default_context['scopes']) == str:
            self.default_context['scopes'] = self.default_context['scopes'].split(',')
        self._data['zones'] = self.get_data('gcp_zones').split(',')
        self.default_context['project'] = self.get_data('gcp_project')

        context = {
            'name': 'default',
            'create_keys': False,
            'cloud': 'gcp',
            'infrastructure_bucket': self._infrastructure_bucket,
            'gcp_zone': self.get_data('zones')[0],
        }

        self._context = merge_dict(self._data, merge_dict(context, self.default_context))

    def add_working_cluster(self):
        context = self.default_context
        context.update({
            'labels': 'cluster=working',
            'zone': self.get_data('zones')[0],
            'node_locations': self.get_data('zones'),
            'cluster_name': self.local_defaults.get('working_cluster_name'),
            'cluster_ipv4_cidr': self.PentagonDefaults.kubernetes['working_cluster_ipv4_cidr']
            })
        gcp.Cluster(context).add('{}/inventory/default/clusters/working'.format(self._repository_directory))

    def add_production_cluster(self):
        context = self.default_context
        context.update({
            'labels': 'cluster=production',
            'zone': self.get_data('zones')[0],
            'locations': self.get_data('zones'),
            'cluster_name': self.local_defaults.get('production_cluster_name'),
            'cluster_ipv4_cidr': self.PentagonDefaults.kubernetes['production_cluster_ipv4_cidr']
            })
        gcp.Cluster(context).add('{}/inventory/default/clusters/production'.format(self._repository_directory))

    def configure_default_project(self):
        inventory.Inventory(self._context).add('{}/inventory/default'.format(self._repository_directory))
        self.add_working_cluster()
        self.add_production_cluster()




import pentagon
from pentagon import migration
from collections import OrderedDict
from pentagon.migration import *


class Migration(migration.Migration):
    _starting_version = '1.2.0'
    _ending_version = '2.0.0'

    def run(self):

        # Create inventory directory if it exists
        inventory_dir = '{}/inventory'.format(self._infrastructure_repository)
        if not os.path.isdir(inventory_dir):
            os.mkdir(inventory_dir)

        # Move default
        if os.path.exists('{}/default'.format(self._infrastructure_repository)):
            self.move('default', "inventory/default")

        self.delete('config/local/env-vars.sh')
        if os.path.exists('{}/config'.format(self._infrastructure_repository)):
            self.move('config/', 'inventory/default/config')

        for item in self.inventory:

            inventory_path = "inventory/{}".format(item)
            logging.debug('Inventory Path: {}'.format(inventory_path))

            if os.path.exists('{}/account/vars.yml'.format(self._infrastructure_repository)):
                account_vars_yml = self.YamlEditor('{}/account/vars.yml'.format(inventory_path)).get()
            else:
                account_vars_yml = OrderedDict()

            if os.path.exists('{}/account/vars.sh'.format(self._infrastructure_repository)):
                account_vars_sh = self.get_file_content('{}/account/vars.sh'.format(inventory_path)).get()
                account_vars = OrderedDict()

                for line in account_vars_sh.split('\n'):
                    line = line.replace('export ', '')
                    llist = line.split('=', 1)
                    account_vars[llist[0]] = llist[1]
            else:
                account_vars = OrderedDict()

            config_vars_yml = self.YamlEditor('{}/config/local/vars.yml'.format(inventory_path))
            config_vars_yml.update(account_vars_yml)
            config_vars_yml.update(account_vars)

            config_vars_yml['ANSIBLE_CONFIG'] = '${{INFRASTRUCTURE_REPO}}/inventory/{}/config'.format(item)
            config_vars_yml['KUBECONFIG'] = "${{INFRASTRUCTURE_REPO}}/inventory/{item}/config/private/kubeconfig".format(item=item)

            config_vars_yml.write()
            self.delete('{}/account'.format(inventory_path))

            if os.path.exists("inventory/{item}/config/local/1password.yml".format(item=item)):
                with self.YamlEditor("inventory/{item}/config/local/1password.yml".format(item=item)) as secrets_yml:
                    secrets_yml['path'] = "inventory/{item}/config/private/".format(item=item)
                    secrets_yml.write()

            # fix ansible path vars
            for file in ['vpn.yml', 'destroy.yml']:
                p = "{}/resources/admin-environment/{}".format(inventory_path, file)
                if os.path.exists("{}/{}".format(self._infrastructure_repository, p)):
                    c = self.get_file_content(p)
                    new_c = c.replace('../../account/vars.yml', '../../config/local/vars.yml')
                    self.overwrite_file(p, new_c)

            local_config_init = '''
if [ -z "${{INFRASTRUCTURE_REPO}}" ]; then
  echo "INFRASTRUCTURE_REPO environment variable must be set"
  exit 1
elif [ ! -d "${{INFRASTRUCTURE_REPO}}" ]; then
  echo "${{INFRASTRUCTURE_REPO}} doesn't exist or isn't a directory"
  exit 1
fi

cd "${{INFRASTRUCTURE_REPO}}/inventory/{item}/config/local" || exit 1

for default_file in *-default; do
  out_file="../private/${{default_file//-default}}"
  echo -n "${{default_file}} -> ${{out_file}} "
  if [ -e "${{out_file}}" ]; then
    echo "already exists. skipping."
    continue
  else
    cat "${{default_file}}" | sed -e "s@__INFRA_REPO_PATH__@$INFRASTRUCTURE_REPO@g" > "${{out_file}}"
    echo "created."
  fi
done
'''.format(item=item)

            self.overwrite_file('{}/config/local/local-config-init'.format(inventory_path), local_config_init, True)

            ansible_cfg_default = '''
[defaults]
inventory = $INFRASTRUCTURE_REPO/plugins/inventory
roles_path = $INFRASTRUCTURE_REPO/roles
filter_plugins = $INFRASTRUCTURE_REPO/plugins/filter_plugins
retry_files_save_path = ~/.ansible-retry
hash_behavior = merge

[ssh_connection]
# this needs the path defined without the use of ENV variables
ssh_args = -F __INFRA_REPO_PATH__/inventory/{item}/config/private/ssh_config
'''.format(item=item)
            self.overwrite_file('{}/config/local/ansible.cfg-default'.format(inventory_path), ansible_cfg_default)

            ssh_config_default = '''
# for the kube / kops working instances
Host 172.20.64.* 172.20.65.* 172.20.66.* 172.20.67.* 172.20.68.* 172.20.69.* 172.20.70.* 172.20.71.* 172.20.72.* 172.20.73.* 172.20.74.* 172.20.75.*
  User admin
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/working_kube
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null

# for the kube / kops prod instances
Host 172.20.96.* 172.20.97.* 172.20.98.* 172.20.99.* 172.20.100.* 172.20.101.* 172.20.102.* 172.20.103.* 172.20.104.* 172.20.105.* 172.20.106.* 172.20.107.*
  User admin
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/production_kube
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null

# for instances in private_working
Host 172.20.48.* 172.20.49.* 172.20.50.* 172.20.51.* 172.20.52.* 172.20.53.* 172.20.54.* 172.20.55.* 172.20.56.* 172.20.57.* 172.20.58.* 172.20.59.*
  User ubuntu
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/working_private
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null

# for instances in private_prod
Host 172.20.32.* 172.20.33.* 172.20.34.* 172.20.35.* 172.20.36.* 172.20.37.* 172.20.38.* 172.20.39.* 172.20.40.* 172.20.41.* 172.20.42.* 172.20.43.*
  User ubuntu
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/production_private
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null

# for instances in admin
Host 172.20.0.* 172.20.1.* 172.20.2.* 172.20.3.* 172.20.4.* 172.20.5.* 172.20.6.* 172.20.7.* 172.20.8.* 172.20.9.* 172.20.10.* 172.20.11.*
  User ubuntu
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/admin_private
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null

# VPN instance
# Replace the '*' with the IP address of the VPN instance
Host *
  User ubuntu
  IdentityFile __INFRA_REPO_PATH__/inventory/{item}/config/private/admin_vpn
  StrictHostKeyChecking no
  UserKnownHostsFile=/dev/null
'''.format(item=item)
        self.overwrite_file('{}/config/local/ssh_config-default'.format(inventory_path), ssh_config_default)
        # update core

        self.delete('{}/config/local/ansible.cfg'.format(inventory_path))
        self.delete('{}/config/local/ssh_config'.format(inventory_path))
        self.delete('tests')
        self.delete('tasks')
        self.delete('components')
        pentagon.component.core.Core({}).add(self._infrastructure_repository, overwrite=True)




from pentagon import migration
from pentagon.migration import *


class Migration(migration.Migration):
    _starting_version = '2.0.0'
    _ending_version = '2.1.0'

    def run(self):

        for item in self.inventory:

            inventory_path = "inventory/{}".format(item)
            logging.debug('Inventory Path: {}'.format(inventory_path))

            with self.YamlEditor('{}/config/local/vars.yml'.format(inventory_path)) as vars_yml:
                if not vars_yml.get('HELM_HOME'):
                    vars_yml['HELM_HOME'] = '${INFRASTRUCTURE_REPO}/helm'
                    vars_yml.write()

            self.delete('roles')



#!/usr/bin/env python
import sys
import os
import click
import logging
import coloredlogs
import traceback
import oyaml as yaml
import json

import pentagon
import migration

from pydoc import locate
from pentagon import PentagonException
from helpers import merge_dict

from meta import __version__, __author__


class RequiredIf(click.Option):

    def __init__(self, *args, **kwargs):
        self.required_if = kwargs.pop('required_if').split('=')
        self.required_option = self.required_if[0]
        self.required_value = self.required_if[1]
        assert self.required_if, "'required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
                          ' NOTE: This argument required when --%s=%s' %
                          (self.required_option, self.required_value)
                          ).strip()
        super(RequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        other_present = self.required_option in opts
        if not other_present or opts[self.required_option] != self.required_value:
            self.prompt = None

        return super(RequiredIf, self).handle_parse_result(
            ctx, opts, args)


@click.group()
@click.version_option(__version__)
@click.option('--log-level', default="INFO", help="Log Level DEBUG,INFO,WARN,ERROR")
@click.pass_context
def cli(ctx, log_level, *args, **kwargs):
    coloredlogs.install(level=log_level)


@click.command()
@click.pass_context
@click.argument('name')
# General directory and file name options
@click.option('-f', '--config-file', help='File to read configuration options from. File supercedes command line options.')
@click.option('-o', '--output-file', default='config.yml', help='File to write options to after completion')
@click.option('--workspace-directory', help='Directory to place new project, defaults to ./')
@click.option('--configure/--no-configure', default=True, help='Configure project with default settings')
@click.option('--force/--no-force', help="Ignore existing directories and copy project")
@click.option('--cloud', default="aws", help="Cloud providor to create default inventory. Defaults to 'aws'. [aws,gcp,none]")
@click.option('--hash-type', default="aws", type=click.Choice(['aws', 'gcp']), help="Type cloud project to create. Defaults to 'aws'")
# General Cloud Options
@click.option('--zones', help="Availability zones as a comma delimited list", cls=RequiredIf, required_if='cloud=gcp')
# Currently only AWS but maybe we can/should add GCP later
@click.option('--configure-vpn/--no-configure-vpn', default=True, help="Whether or not to configure the vpn.")
@click.option('--vpc-name', help="Name of VPC to create")
@click.option('--vpc-cidr-base', help="First two octets of the VPC ip space")
@click.option('--vpc-id', help="AWS VPC id to create the kubernetes clusters in")
@click.option('--admin-vpn-key', help="Name of the ssh key for the admin user of the VPN instance")
@click.option('--vpn-ami-id', help="ami-id to use for the VPN instance")
# General Kubernetes options
@click.option('--kubernetes-version', help="Version of kubernetes to use for cluster nodes")
@click.option('--disk-size', help="Size disk to provision on the kubernetes vms")
# Working
@click.option('--working-kubernetes-cluster-name', help="Name of the working kubernetes cluster nodes")
@click.option('--working-kubernetes-node-count', help="Name of the working kubernetes cluster nodes")
@click.option('--working-kubernetes-worker-node-type', help="Node type of the kube workers")
@click.option('--working-kubernetes-network-cidr', help="Network cidr of the kubernetes working cluster")
# Production
@click.option('--production-kubernetes-cluster-name', help="Name of the production kubernetes cluster nodes")
@click.option('--production-kubernetes-node-count', help="Name of the production kubernetes cluster nodes")
@click.option('--production-kubernetes-worker-node-type', help="Node type of the kube workers")
@click.option('--production-kubernetes-network-cidr', help="Network cidr of the kubernetes working cluster")
# AWS Cloud options
@click.option('--aws-access-key', prompt=True, default=lambda: os.environ.get('PENTAGON_aws_access_key'), help="AWS access key", cls=RequiredIf, required_if='cloud=aws')
@click.option('--aws-secret-key', prompt=True, default=lambda: os.environ.get('PENTAGON_aws_secret_key'), help="AWS secret key", cls=RequiredIf, required_if='cloud=aws')
@click.option('--aws-default-region', help="AWS default region", cls=RequiredIf, required_if='cloud=aws')
@click.option('--aws-availability-zones', help="[Deprecated] Use \"--availability-zones\". AWS availability zones as a comma delimited with spaces. Default to region a, region b, ... region z")
@click.option('--aws-availability-zone-count', help="Number of availability zones to use")
@click.option('--infrastructure-bucket', help="Name of S3 Bucket to store state")
@click.option('--dns-zone', help="DNS zone to configure DNS records in")
@click.option('--create-keys/--no-create-keys', default=True, help="Create ssh keys or not")
# AWS only Kubernetes options
# Working
@click.option('--working-kubernetes-master-aws-zone', help="Availability zone to place the kube master in")
@click.option('--working-kubernetes-master-node-type', help="AWS only. Node type of the kube master")
@click.option('--working-kube-key', help="Name of the ssh key for the working kubernetes cluster")
@click.option('--working-private-key', help="Name of the ssh key for the working non kubernetes instances")
@click.option('--working-kubernetes-dns-zone', help="DNS Zone of the kubernetes working cluster")
@click.option('--working-kubernetes-v-log-level', help="V Log Level kubernetes working cluster")
# Poduction
@click.option('--production-kubernetes-master-aws-zone', help="Availability zone to place the kube master in")
@click.option('--production-kubernetes-master-node-type', help=" AWS only. Node type of the kube master")
@click.option('--production-kube-key', help="Name of the ssh key for the production kubernetes cluster")
@click.option('--production-private-key', help="Name of the ssh key for the production non kubernetes instances")
@click.option('--production-kubernetes-dns-zone', help="DNS Zone of the kubernetes production cluster")
@click.option('--production-kubernetes-v-log-level', help="V Log Level kubernetes production cluster")
# GCP Cloud options
@click.option('--gcp-project', prompt=True, help="Google Cloud Project to create clusters in", cls=RequiredIf, required_if='cloud=gcp')
@click.option('--gcp-zones', prompt=True, help="Google Cloud Project zones to create clusters in. Comma separated list.", cls=RequiredIf, required_if='cloud=gcp')
@click.option('--gcp-region', prompt=True, help="Google Cloud Region to create regional resources in", cls=RequiredIf, required_if='cloud=gcp')
def start_project(ctx, name, **kwargs):
    """ Create an infrastructure project from scratch with the configured options """

    try:

        logging.basicConfig(level=kwargs.get('log_level'))
        file_data = {}
        if kwargs.get('config_file'):
            file_data = parse_infile(kwargs.get('config_file'))[0]
        kwargs.update(file_data)
        logging.debug(kwargs)
        cloud = kwargs.get('cloud')
        if cloud.lower() == 'aws':
            project = pentagon.AWSPentagonProject(name, kwargs)
        elif cloud.lower() == 'gcp':
            project = pentagon.GCPPentagonProject(name, kwargs)
        elif cloud.lower() == 'none':
            project = pentagon.PentagonProject(name, kwargs)
        else:
            raise PentagonException("Value passed for option --cloud not 'aws' or 'gcp'")
        logging.debug('Creating {} project {} with {}'.format(cloud.upper(), name, kwargs))
        project.start()
    except Exception as e:
        logging.error(e)
        logging.debug(traceback.format_exc(e))


@click.command()
@click.pass_context
@click.argument('component_path')
@click.option('--data', '-D', multiple=True, help='Individual Key=Value pairs used by the component. There should be no spaces surrounding the `=`')
@click.option('--file', '-f', help='File to read Key=Value pair from (yaml or json are supported)')
@click.option('--out', '-o', default='./', help="Path to output module result, if any")
@click.argument('additional-args', nargs=-1, default=None)
def add(ctx, component_path, additional_args, **kwargs):
    _run('add', component_path, additional_args, kwargs)


@click.command()
@click.pass_context
@click.argument('component_path')
@click.option('--data', '-D', multiple=True, help='Individual Key=Value pairs used by the component')
@click.option('--file', '-f', help='File to read Key=Value pair from (yaml or json are supported)')
@click.option('--out', '-o', default='./', help="Path to output module result, if any")
@click.argument('additional-args', nargs=-1, default=None)
def get(ctx, component_path, additional_args, **kwargs):
    _run('get', component_path, additional_args, kwargs)


@cli.command()
@click.pass_context
@click.option("--dry-run/--no-dry-run", default=False, help="Test migration before applying")
@click.option('--log-level', default="INFO", help="Log Level DEBUG,INFO,WARN,ERROR")
@click.option('--branch', default="migration", help="Name of branch to create for migration. Default='migration'")
@click.option('--yes/--no', default=False, help="Confirm to run migration")
def migrate(ctx, **kwargs):
    """ Update Infrastructure Repository to the latest configuration """
    logging.basicConfig(level=kwargs.get('log_level'))
    migration.migrate(kwargs['branch'], kwargs['yes'])


def _run(action, component_path, additional_args, options):
    logging.basicConfig(level=options.get('log_level'))
    logging.debug("Importing module Pentagon {}".format(component_path))
    logging.debug("with options: {}".format(options))
    logging.debug("and additional arguments: {}".format(additional_args))

    documents = [{}]
    data = parse_data(options.get('data', {}))

    try:
        file = options.get('file', None)
        if file is not None:
            documents = parse_infile(file)
    except Exception as e:
        logging.error("Error parsing data from file or -D arguments")
        logging.error(e)

    component_class = get_component_class(component_path)
    try:
        for doc in documents:
            if callable(component_class):
                data = merge_dict(doc, data, clobber=True)
                # Making data keys more flexible and allowing keys with
                # - to be be corrected in place
                data_copy = data.copy()
                for key, value in data.iteritems():
                    flex_key = key.replace('-', '_')
                    if flex_key != key:
                        data_copy[flex_key] = value
                data = data_copy

                getattr(component_class(data, additional_args), action)(options.get('out'))
            else:
                logging.error("Error locating module or class: {}".format(component_path))
    except Exception, e:
        logging.error(e)
        logging.debug(traceback.format_exc(e))


# Making names more terminal friendly
cli.add_command(start_project, "start-project")
cli.add_command(add, "add")
cli.add_command(get, "get")


def get_component_class(component_path):
    """ Construct Class path from component input """

    component_path_list = component_path.split(".")
    possible_component_paths = []
    if len(component_path_list) > 1:
        component_name = ".".join(component_path.split(".")[0:-1])
        component_class_name = component_path.split(".")[-1]
    else:
        component_name = component_path
        component_class_name = component_path

    # Compile list of possible class paths
    possible_component_paths.append('{}.{}'.format(component_name, component_class_name))
    possible_component_paths.append('{}.{}'.format(component_name, component_class_name.title()))
    possible_component_paths.append('pentagon.component.{}.{}'.format(component_name, component_class_name))
    possible_component_paths.append('pentagon.component.{}.{}'.format(component_name, component_class_name.title()))
    possible_component_paths.append('pentagon_{}.{}'.format(component_name, component_class_name))
    possible_component_paths.append('pentagon_{}.{}'.format(component_name, component_class_name.title()))

    # Find Class if it exists
    for class_path in possible_component_paths:
        logging.debug('Seeking {}'.format(class_path))
        component_class = locate(class_path)
        if component_class is not None:
            logging.debug("Found {}".format(component_class))
            return component_class

        logging.debug('{} Not found'.format(class_path))


def parse_infile(file):
    """ Parse data structure from file into dictionary for component use """
    with open(file, 'r') as data_file:
        try:
            data = json.load(data_file)
            logging.debug("Data parsed from file {}: {}".format(file, data))
            return data
        except ValueError as json_error:
            pass

        data_file.seek(0)

        try:
            data = list(yaml.load_all(data_file, Loader=yaml.loader.BaseLoader))
            logging.debug("Data parsed from file {}: {}".format(file, data))
            return data
        except yaml.YAMLError as yaml_error:
            pass

    logging.error("Unable to parse in file. {} {} ".format(json_error, yaml_error))


def parse_data(data, d=None):
    """ Function to parse the incoming -D options into a dict """
    if d is None:
        d = {}

    for kv in data:
        key = kv.split('=')[0]
        try:
            val = kv.split('=', 1)[1]
        except IndexError, e:
            val = True

        d[key] = val

    return d



#!/usr/bin/env python
# -- coding: utf-8 --
# Copyright 2017 Reactive Ops Inc.
#
# Licensed under the Apache License, Version 2.0 (the “License”);
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an “AS IS” BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
from setuptools import setup, find_packages
from pentagon import meta

try:
    from setuptools import setup, find_packages
except ImportError:
    print("setup tools required. Please run: "
          "pip install setuptools).")
    sys.exit(1)


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths

extra_files = package_files('pentagon/component')

setup(name='pentagon',
      version=meta.__version__,
      description='Radically simple kubernetes',
      author=meta.__author__,
      author_email='services@reactiveops.com',
      url='http://reactiveops.com/',
      license='Apache2.0',
      include_package_data=True,
      install_requires=[
        "click==6.7",
        "GitPython==2.1.3",
        "Jinja2==2.9.5",
        "pycrypto==2.6.1",
        "oyaml>=0.5",
        "shyaml==0.5.0",
        "ansible==2.5.2",
        "boto3==1.7.16",
        "botocore==1.10.29",
        "boto==2.49.0",
        "google-api-python-client==1.6.2",
        "yamlord==0.4",
        "coloredlogs==9.0",
        "awscli==1.15.29",
        "semver>=2.8.0",
      ],
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: Information Technology',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: Apache Software License',
          'Natural Language :: English',
          'Operating System :: POSIX',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Topic :: System :: Installation/Setup',
          'Topic :: System :: Systems Administration',
          'Topic :: Utilities',
      ],
      entry_points=''' #for click integration
          [console_scripts]
          pentagon=pentagon.cli:cli
      ''',
      packages=find_packages(exclude=['tests', 'example-component']),
      scripts=[
         'bin/yaml_source',
         ],
      #package_data={'': extra_files},
      data_files=[],
      )



import datetime


class AWSPentagonDefaults(object):
    ssh = {
        'admin_vpn_key': 'admin-vpn',
        'working_kube_key': 'working-kube',
        'working_private_key': 'working-private',
        'production_kube_key': 'production-kube',
        'production_private_key': 'production-private',
    }

    kubernetes = {
        'node_count': 3,
        'master_node_type': 't2.medium',
        'master_node_image': '379101102735/debian-stretch-hvm-x86_64-gp2-2018-10-01-66564',
        'worker_node_type': 't2.medium',
        'worker_node_image': '379101102735/debian-stretch-hvm-x86_64-gp2-2018-10-01-66564',
        'node_root_volume_size': 200,
        'v_log_level': 10,
        'network_cidr': '172.20.0.0/16',
        'kubernetes_version': '1.10.8',
        'network_mask': 24,
        'third_octet': 16,
        'production_third_octet': 16,
        'working_third_octet': 24,
        'third_octet_increment': 1,
        'authorization': {'rbac': {}},
        'networking': {'flannel': {'backend': 'vxlan'}},
        'node_additional_policies': '[{"Effect": "Allow","Action": ["autoscaling:DescribeAutoScalingGroups", "autoscaling:DescribeAutoScalingInstances", "autoscaling:DescribeTags", "autoscaling:SetDesiredCapacity", "autoscaling:TerminateInstanceInAutoScalingGroup"],"Resource": "*"}]',
        'ssh_key_path': '~/.ssh/id_rsa.pub'
    }

    vpc = {
        'vpc_name': datetime.datetime.today().strftime('%Y%m%d'),
        'vpc_cidr_base': '172.20',
        'aws_availability_zone_count': 3,
    }


class GCPPentagonDefaults(object):

    kubernetes = {
        'cluster_version': "1.8.7-gke.1",
        'enable_basic_auth': False,
        'machine_type': 'n1-standard-2',
        'image_type': 'COS',
        'num_nodes': 1,
        'working_cluster_ipv4_cidr': '172.20.0.0/16',
        'production_cluster_ipv4_cidr': '172.21.0.0/16',
        'network': 'default',
        'subnetwork': 'default',
        'enable_autoscaling': True,
        'min_nodes': 1,
        'max_nodes': 3,
        'enable_autoupgrade': True,
        'scopes': [
            'https://www.googleapis.com/auth/compute',
            'https://www.googleapis.com/auth/devstorage.read_only',
            'https://www.googleapis.com/auth/logging.write',
            'https://www.googleapis.com/auth/monitoring',
            'https://www.googleapis.com/auth/servicecontrol',
            'https://www.googleapis.com/auth/service.management.readonly',
            'https://www.googleapis.com/auth/trace.append'
        ]

    }



import os
import glob
import shutil
import logging
import traceback
import sys
import re
import subprocess
import yaml

from pentagon.component import ComponentBase
from pentagon.helpers import render_template
from pentagon.defaults import AWSPentagonDefaults as PentagonDefaults


class Cluster(ComponentBase):
    _path = os.path.dirname(__file__)

    def add(self, destination):
        for key in PentagonDefaults.kubernetes:
            if not self._data.get(key):
                self._data[key] = PentagonDefaults.kubernetes[key]

        if not self._data.get('network_cidr_base'):
            self._data['network_cidr_base'] = PentagonDefaults.vpc['vpc_cidr_base']

        for key in ['authorization', 'networking']:
            self._data[key] = yaml.dump(self._data[key])

        return super(Cluster, self).add(destination)

    def get(self, destination):

        self._cluster_name = self._data.get('name', os.environ.get('CLUSTER_NAME'))
        self._bucket = self._data.get('kops_state_store', os.environ.get('KOPS_STATE_STORE'))
        self._destination = destination

        if self._bucket is None:
            logging.error("kops_state_store required.")
            sys.exit(1)

        if self._cluster_name is None:
            logging.error("name is required.")
            sys.exit(1)

        os.mkdir(self._cluster_name)
        os.chdir(self._cluster_name)

        self._get_cluster_yaml()

        for ig in self._cluster_instance_groups:
            self._get_instance_group_yaml(ig)

        self._get_cluster_admin_secret()

    @property
    def _cluster_instance_groups(self):
        # get igs yaml
        logging.debug("Getting instance groups.")
        args = ['kops',
                'get',
                'ig',
                '--name={}'.format(self._cluster_name),
                '--state=s3://{}'.format(self._bucket)]

        return [ig.split("\t")[0] for ig in subprocess.check_output(args).split("\n")][1:-1]

    def _get_instance_group_yaml(self, ig):
        args = ['kops',
                'get',
                'ig',
                ig,
                '--name={}'.format(self._cluster_name),
                '--state=s3://{}'.format(self._bucket),
                '-oyaml']

        ig_yaml = subprocess.check_output(args)

        file_mode = 'w'
        if "master" in ig:
            ig_file_name = "master.yml"
            file_mode = 'a'
        else:
            ig_file_name = "{}.yml".format(ig)

        with open(ig_file_name, file_mode) as ig_file:
            ig_file.write("---\n")
            ig_file.write("{}\n".format(ig_yaml))
            ig_file.close()

    def _get_cluster_admin_secret(self):
        # get secret sorta
        logging.debug("Getting ssh key secret. This will require transformation before a new cluster can be created")
        with open('secret.sh', 'w') as sf:
            args = ['kops',
                    'get',
                    'secret',
                    'admin',
                    '--name={}'.format(self._cluster_name),
                    '--state=s3://{}'.format(self._bucket)]

            subprocess.Popen(args, stdout=sf)

    def _get_cluster_yaml(self):
        # get cluster yaml
        logging.debug("Getting cluster.")
        with open('cluster.yml', 'w') as cf:
            args = ['kops',
                    'get',
                    'cluster',
                    '--name={}'.format(self._cluster_name),
                    '--state=s3://{}'.format(self._bucket),
                    '-oyaml']

            p = subprocess.Popen(args, stdout=cf)
            stdout, stderr = p.communicate()
            if p.returncode != 0:
                logging.error("Error getting cluster: {}".format(stderr))
                sys.exit(1)




from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict


class Migration(migration.Migration):
    _starting_version = '2.1.0'
    _ending_version = '2.2.0'

    def run(self):

        # Add new versions of files
        c = core.Core({'cloud': 'aws'})
        c._overwrite = True
        c._destination = "./Makefile.jinja"
        c._add_files('Makefile.jinja')
        c._render_directory_templates()


        for item in self.inventory:

            inventory_path = "inventory/{}".format(item)
            logging.debug('Inventory Path: {}'.format(inventory_path))

            with self.YamlEditor('{}/config/local/vars.yml'.format(inventory_path)) as vars_yml:
                inventory_vars = vars_yml.get_data()

            template_context  = {
                'aws_region': inventory_vars.get('AWS_DEFAULT_REGION'),
                'infrastructure_bucket': inventory_vars.get('INFRASTRUCTURE_BUCKET')
            }

            if os.path.exists("{}/vpc".format(inventory_path)):
                # Move files around
                self.move("{}/vpc/".format(inventory_path), "{}/terraform/".format(inventory_path))
                self.move("{}/terraform/terraform.tfvars".format(inventory_path), "{}/terraform/aws_vpc.auto.tfvars".format(inventory_path))
                self.move("{}/terraform/variables.tf".format(inventory_path), "{}/terraform/aws_vpc_variables.tf".format(inventory_path))
                self.move("{}/terraform/main.tf".format(inventory_path), "{}/terraform/aws_vpc.tf".format(inventory_path))

                # Mutate files
                aws_vpc_file_content = self.get_file_content("{}/terraform/aws_vpc.tf".format(inventory_path)).split('\n')
                new_aws_vpc_file_content = aws_vpc_file_content[:]
                if '// terraform backend config' in aws_vpc_file_content:
                    i = aws_vpc_file_content.index("// terraform backend config")
                    # Should remove provider and backend config if present
                    new_aws_vpc_file_content = \
                        aws_vpc_file_content[0:i-1] + \
                        aws_vpc_file_content[i+9:]

                self.delete("{}/terraform/terraform-remote.sh".format(inventory_path))

                new_aws_vpc_file_content = ('\n').join(new_aws_vpc_file_content[6:-1])
                self.overwrite_file("{}/terraform/aws_vpc.tf".format(inventory_path), new_aws_vpc_file_content)

                # Add new versions of files
                i = inventory.Inventory(merge_dict(template_context, {'cloud': 'aws', 'name': item}))
                i._overwrite = True
                i._destination = "{}/terraform/".format(inventory_path)
                i._add_files('terraform/backend.tf.jinja')
                i._add_files('terraform/Makefile.jinja')
                i._add_files('terraform/provider.tf.jinja')
                i._render_directory_templates()

                logging.warn("####### IMPORTANT: Your s3 backend configuration as changed ######")
                logging.warn("Move your state path in s3:")
                logging.warn("Command example: (only for example purposes) ")
                logging.warn("aws s3 sync s3://{bucket}/{vpc_tag_name}/{old_path}/ s3://{bucket}/{item}/{new_path}/".format(
                    bucket=inventory_vars.get('INFRASTRUCTURE_BUCKET'),
                    vpc_tag_name=inventory_vars.get('vpc_tag_name'),
                    item=item,
                    old_path='terraform-vpc',
                    new_path='terraform')
                )
                logging.warn("aws s3 rm s3://{bucket}/{org}/{old_path}/".format(
                    bucket=inventory_vars.get('INFRASTRUCTURE_BUCKET'),
                    org=inventory_vars.get('org_name'),
                    old_path='terraform-vpc')
                )




import oyaml as yaml

from pentagon import migration
from pentagon.migration import *
from pentagon.component import core, inventory
from pentagon.helpers import merge_dict


class Migration(migration.Migration):
    _starting_version = '2.4.3'
    _ending_version = '2.5.0'

    def run(self):

        # Remove Orgname from vars.yml
        # Replace org-name with org in all vpn files
        # Remove 'org' arg for vpn role call  in vpn.yml
        for item in self.inventory:
            inventory_path = "inventory/{}".format(item)
            with self.YamlEditor('{}/config/local/vars.yml'.format(inventory_path)) as vars_yml:
                vars_yml.remove('org-name')
                vars_yml.remove('secrets_bucket')
                vars_yml.write()

            if os.path.exists("{}/resources/admin-environment/".format(inventory_path)):
                with self.YamlEditor("{}/resources/admin-environment/vpn.yml".format(inventory_path)) as vpn_yml:
                    data = vpn_yml.get_data()
                    del data[2]['roles'][0]['org']

                    self.overwrite_file("{}/resources/admin-environment/vpn.yml".format(inventory_path), yaml.safe_dump(data, default_flow_style=False))

                with self.YamlEditor("{}/resources/admin-environment/env.yml".format(inventory_path)) as env_yml:
                    env_yml['env'] = "{{ org }}"
                    env_yml['open_vpn_host'] = "vpn-{{ org }}.{{ canonical_zone }}"
                    env_yml.write()

                with self.YamlEditor("{}/resources/admin-environment/env.yml".format(inventory_path)) as env_yml:
                    data = env_yml.get_data()
                    del data['vpn_bucket']
                    self.overwrite_file("{}/resources/admin-environment/env.yml".format(inventory_path), yaml.safe_dump(data, default_flow_style=False))




import logging
import os
import shutil
import distutils.dir_util
import sys
import glob
import git
import inspect
import yamlord
import semver
import fnmatch

from collections import OrderedDict

from pentagon.migration import migrations
from pentagon.pentagon import PentagonException
from pentagon.helpers import write_yaml_file
from pentagon.meta import __version__ as pentagon_version

from pydoc import locate
from distutils.version import StrictVersion

default_version = "1.2.0"
version_file = '.version'


def migrate(branch='migration', yes=False):
    """ Find applicabale migrations and run them """
    logging.info("Pentagon Version: {}".format(installed_version()))
    logging.info("Starting Repository Version: {}".format(current_version()))

    migrations = migrations_to_run(current_version(), available_migrations())
    if migrations:
        logging.info("There are Migrations to run: ")
        logging.info(migrations)
        if yes:
            for migration in migrations:
                logging.info('Starting migration: {}'.format(migration))
                migration_name = "migration_{}".format(migration.replace('.', '_'))
                migration_class = locate("pentagon.migration.migrations.{}".format(migration_name))
                migration_class.Migration(branch).start()
            logging.info("Migrations complete. Use git to merge the migration branch.")
            logging.info("Current Repository Version: {}".format(current_version()))
        else:
            logging.info("Use: `pentagon migrate --yes` to run migrations")
    else:
        logging.info("No Migrations to run.")
        compare_value = semver.compare(installed_version(), current_version())
        if compare_value == -1:
            logging.error("Repository Version > Installed Version. Upgrade Pentagon")
            sys.exit(1)
        elif compare_value == 1:
            logging.info("Installed Version > Repository Version.")
            logging.info(" Use `pentagon migrate --yes` to update Repository Version")
            if yes:
                Migration(None).version_only()
        elif compare_value == 0:
            logging.info("You are at the latest version!")


def migrations_to_run(current_version, available_migrations):
    m = [v for v in available_migrations if StrictVersion(v) >= StrictVersion(current_version)]
    logging.debug("Migrations to run: {}".format(m))
    return m


def available_migrations():
    """ Gets and returns a list of migration modules """
    m = []
    for file in glob.glob("{}/migration_*.py".format(migrations.__path__[0])):
        m.append(os.path.basename(os.path.splitext(file)[0]).replace('migration_', '').replace('_', '.'))
        logging.debug("Migration Found: {}".format(file))
    m.sort(key=StrictVersion)
    logging.debug("Available Migrations: {}".format(m))
    return m


def installed_version():
    """ get installed version of pentagon """
    return pentagon_version


def infrastructure_repository():
    infrastructure_repo = os.environ.get('INFRASTRUCTURE_REPO')
    if infrastructure_repo is None:
        raise PentagonException('Required environnment variable INFRASTRUCTURE_REPO is not set.')
    return infrastructure_repo


def current_version(version_file=version_file):
    """ get current version of the infrastucture_repo """
    try:
        with open("{}/{}".format(infrastructure_repository(), version_file)) as vf:
            version = vf.readline()
    except IOError:
        logging.warn("{} not found. Using default version {}".format(version_file, default_version))
        version = default_version

    return version


class Migration(object):
    """ Parent class for pentagon migrations """

    class YamlEditor(object):

        def __init__(self, file):
            # Fetch yaml file as ordered dict
            self.file = file
            self.data = yamlord.read_yaml(self.file)
            logging.debug(self.data)

        def update(self, new_data, clobber=False):
            """ accepts a dict and appends keys to ordered dict. Updates keys if clobber is True"""
            nd = OrderedDict(new_data)
            self.data.update(nd)

        def remove(self, keys):
            """ accepts a list of keys to remove from yaml """
            for key in keys:
                if key in self.data.keys():
                    del self.data[key]

        def get_data(self):
            """ return orderd dict of yaml """
            return self.data

        def write(self, file=None):
            if file is not None:
                self.file = file
            return yamlord.write_yaml(self.data, self.file)

        def get(self, key, default=None):
            return self.data.get(key, default)

        def __getitem__(self, key):
            return self.data[key]

        def __setitem__(self, key, value):
            self.data[key] = value

        def __str__(self):
            str(self.data)

        def __enter__(self):
            return self

        def __exit__(self, type, value, traceback):
            pass

    def __init__(self, branch_name):
        logging.debug("This got run")
        self._infrastructure_repository = infrastructure_repository()
        self.branch = branch_name

    def start(self):
        """ run migration """
        self._run()

    def version_only(self):
        """ Only increase version in .version_file """
        self.overwrite_file(version_file, installed_version())

    def real_path(self, path):
        return os.path.normpath("{}/{}".format(self._infrastructure_repository, path))

    def _branch(self):
        repo = git.Repo(self._infrastructure_repository)
        new_branch = repo.create_head(self.branch)
        repo.git.checkout(self.branch)

    def _run(self):
        os.chdir(self._infrastructure_repository)
        self._branch()
        self.run()
        self._write_new_version(installed_version())

    def _write_new_version(self, version):
        """ write new file with new version following the migration """
        self.overwrite_file(version_file, version)

    def move(self, source, destination):
        """ move files and directories with extreme predjudice """
        logging.info("Moving {} -> {}".format(self.real_path(source), self.real_path(destination)))

        if os.path.isfile(source):
            _move = shutil.move
        elif os.path.exists(destination):
            from distutils.dir_util import copy_tree as _move
        else:
            from shutil import copytree as _move

        _move(self.real_path(source), self.real_path(destination))
        self.delete(source)

    def overwrite_file(self, path, content, executable=False):
        """ alias create_file """
        self.create_file(path, content, executable)

    def create_file(self, path, content, executable=False):
        """ Create a new file """
        path = "{}/{}".format(self._infrastructure_repository, path)
        with open(path, 'w') as f:
            f.write(content)

        if executable is True:
            mode = os.stat(path).st_mode
            mode |= (mode & 0o444) >> 2    # copy R bits to X
            os.chmod(path, mode)

    def create_dir(self, path):
        """ Recursively create a directory """
        path = "{}/{}".format(self._infrastructure_repository, path)
        try:
            os.makedirs(path)
        except OSError:
            if not os.path.isdir(path):
                raise

    def get_file_content(self, path):
        """ Retreive file contents in a string """
        with open(self.real_path(path), 'r') as f:
            return f.read()

    @property
    def inventory(self, exclude=[]):
        """ Returns list of inventory item, excluding list 'exclude' """
        return [d for d in os.walk("{}/inventory".format(self._infrastructure_repository)).next()[1] if d not in exclude]

    def delete(self, path):
        """ deletes file or directory """
        logging.info("Deleteing {}".format(path))
        if os.path.isfile(self.real_path(path)):
            return os.remove(self.real_path(path))

        if os.path.isdir(self.real_path(path)):
            return shutil.rmtree(self.real_path(path))

        return False

    def find_files(self, path='./', file_pattern=None):
        matches = []
        for root, dirnames, filenames in os.walk(path):
            for filename in fnmatch.filter(filenames, file_pattern):
                matches.append(os.path.join(root, filename))

        if len(matches) == 0:
            logging.warn("No {} files found!".format(file_pattern))
        return matches
